var MessageHeader = System.ServiceModel.Channels.MessageHeader;
var Application = System.Windows.Application;
var CultureInfo = System.Globalization.CultureInfo;
var StringBuilder = System.Text.StringBuilder;
var HtmlElement = System.Windows.Browser.HtmlElement;
var HtmlPage = System.Windows.Browser.HtmlPage;
var StreamWriter = System.IO.StreamWriter;
var IsolatedStorageFileStream = System.IO.IsolatedStorage.IsolatedStorageFileStream;
var IsolatedStorageFile = System.IO.IsolatedStorage.IsolatedStorageFile;
var FileMode = System.IO.FileMode;
var FileAccess = System.IO.FileAccess;
var DateTime = System.DateTime;
var TimeSpan = System.TimeSpan;
var BasicHttpBinding = System.ServiceModel.BasicHttpBinding;
var BasicHttpSecurityMode = System.ServiceModel.BasicHttpSecurityMode;
var List = System.Collections.Generic.List;
var ManualResetEvent = System.Windows.Threading.ManualResetEvent;
var MemoryStream = System.IO.MemoryStream;
var Dictionary = System.Collections.Generic.Dictionary;
var Convert = System.Convert;
var StringComparison = System.StringComparison;
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        /// <summary>
        /// Common methods in Alarm Command.
        /// RSViewSE leave back door for user to pre- input alarm command comments and shelve duration.
        /// And the format for user pre- input alarm commands could be like:
        /// 1. AE_Shelve "AEServer" / CPreComments / D1 / P
        /// 2. AE_Shelve "AEServer" / C"PreComments" / D1 / P
        /// 3. AE_Shelve "AEServer" / C PreComments / D1 / P
        /// 4. AE_Shelve "AEServer" / C "PreComments" / D1 / P
        /// 5. AE_Shelve "AEServer" / D1 / P / CPreComments
        /// 6. AE_Shelve "AEServer" / D1 / P / C"PreComments"
        /// 7. AE_Shelve "AEServer" / D1 / P / C PreComments
        /// 8. AE_Shelve "AEServer" / D1 / P / C "PreComments"
        /// 9. AE_Shelve "AEServer" / D1 / CPreComments / P
        /// 10. AE_Shelve "AEServer" / D1 / C"PreComments" / P
        /// 11. AE_Shelve "AEServer" / D1 / C PreComments / P
        /// 12. AE_Shelve "AEServer" / D1 / C "PreComments" / P
        /// We also need to ignore when / D or / C or / P is in the pre- input comments.
        /// </summary>
        var AlarmCommandUtils = /** @class */ (function () {
            function AlarmCommandUtils() {
            }
            /// <summary>
            /// Determine whether to popup the alarm command dialog.
            /// We need to ignore the "/P" in pre-input comments.
            /// This method does not care about the format validation of the command paramter string
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.PopupAlarmCommandDialog = function (parameter) {
                var nStartIndex = this.FindIndexAfterAlarmSourceServer(parameter);
                if (nStartIndex == -1) {
                    // Parameter invalid
                    return false;
                }
                var nPreLength = nStartIndex;
                var strParameter = parameter.substr(nStartIndex);
                var nPopDialogIndex = this.FindIndexAfterSign(strParameter.toUpperCase(), this.AE_PARAMETER_SIGN_POPUPDIALOG_COMMENTS);
                if (nPopDialogIndex == -1) {
                    return false;
                }
                var nCommentsStartIndex = this.FindCommentsStartIndex(strParameter);
                if (nPopDialogIndex < nCommentsStartIndex || nCommentsStartIndex == -1) {
                    // In this case the command parameter should be like
                    // AE_Remark :AEServer /P /C"user pre-input comments"
                    // or no user pre-input comments
                    return true;
                }
                else {
                    // In some cases it would be like
                    // AE_Remark :AEServer /C"some characters with /P" /P
                    var nCommentsEndIndex = this.FindCommentsEndIndex(strParameter);
                    if (nPopDialogIndex > nCommentsEndIndex) {
                        return true;
                    }
                    nPreLength = nPreLength + nCommentsEndIndex;
                    var strTemp = strParameter.substr(nCommentsEndIndex);
                    nPopDialogIndex = this.FindIndexAfterSign(strTemp.toUpperCase(), this.AE_PARAMETER_SIGN_POPUPDIALOG_COMMENTS);
                    if (nPopDialogIndex == -1) {
                        // In this case the command parameter would be like
                        // AE_Remark :AEServer /C"some characters with /P"
                        return false;
                    }
                }
                return true;
            };
            /// <summary>
            /// Get the alarm source in the parameter string
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.GetAlarmSource = function (parameter) {
                var retVar = null;
                var strTemp = parameter.trim();
                if (strTemp.StartsWith(this.AE_PARAMETER_SIGN_QUOTATIONMARK)) {
                    //The second quotationmark should be the end of alarm source
                    var alarmSourceEndIndex = strTemp.substr(strTemp.IndexOf(this.AE_PARAMETER_SIGN_QUOTATIONMARK) + 1).IndexOf(this.AE_PARAMETER_SIGN_QUOTATIONMARK);
                    retVar = strTemp.substr(1, alarmSourceEndIndex);
                }
                else {
                    //If the alarm source is not quoted then it should contain only one alarm without whitespace
                    var spaceIndex = parameter.trim().IndexOf(this.AE_PARAMETER_SIGN_SPACESLASH);
                    if (spaceIndex > 0) {
                        retVar = parameter.substr(0, spaceIndex);
                    }
                    else {
                        retVar = parameter;
                    }
                }
                return retVar;
            };
            /// <summary>
            /// Get shelve duration in the parameter string
            /// Ignore "/D" in pre-input comments
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.GetShelveDuration = function (parameter) {
                var nStartIndex = this.FindIndexAfterAlarmSourceServer(parameter);
                if (nStartIndex == -1) {
                    // Parameter invalid
                    return null;
                }
                var strParameter = parameter.substr(nStartIndex);
                var retVar = null;
                var nShelveDurationStartIndex = this.FindShelveDurationStartIndex(strParameter);
                if (nShelveDurationStartIndex == -1 || nShelveDurationStartIndex == strParameter.length) {
                    return retVar;
                }
                var nShelveDurationEndIndex = this.FindShelveDurationEndIndex(strParameter);
                if (nShelveDurationStartIndex == nShelveDurationEndIndex) {
                    return retVar;
                }
                retVar = strParameter.substr(nShelveDurationStartIndex, nShelveDurationEndIndex - nShelveDurationStartIndex);
                return retVar;
            };
            /// <summary>
            /// Get the alarm server name in the parameter string for AE Remark
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.GetAlarmServerName = function (parameter) {
                var retVar = null;
                var strTemp = parameter.trim();
                if (strTemp.StartsWith(this.AE_PARAMETER_SIGN_QUOTATIONMARK)) {
                    //alarm server name contains space
                    var endIndex = strTemp.substr(1).IndexOf(this.AE_PARAMETER_SIGN_QUOTATIONMARK);
                    retVar = strTemp.substr(1, endIndex);
                }
                else {
                    //If the alarm server is not quoted then it should contain only one server
                    var spaceIndex = strTemp.IndexOf(this.AE_PARAMETER_SIGN_SPACESLASH);
                    if (spaceIndex > 0) {
                        retVar = strTemp.substr(0, spaceIndex);
                    }
                    else {
                        retVar = strTemp;
                    }
                }
                return retVar;
            };
            /// <summary>
            /// Get the pre-input comments in the parameter string.
            /// We do not care about the format validation of the command paramter string
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.GetComments = function (parameter) {
                var nStartIndex = this.FindIndexAfterAlarmSourceServer(parameter);
                if (nStartIndex == -1) {
                    // Parameter invalid
                    return null;
                }
                var nPreLength = nStartIndex;
                var strParameter = parameter.substr(nStartIndex);
                var retVar = null;
                var nCommentsStartIndex = this.FindCommentsStartIndex(strParameter);
                if (nCommentsStartIndex == -1) {
                    return retVar;
                }
                var nCommentsEndIndex = this.FindCommentsEndIndex(strParameter);
                if (nCommentsStartIndex == nCommentsEndIndex) {
                    return retVar;
                }
                if (this.IsQuotationMark(strParameter, nCommentsStartIndex)) {
                    if (nCommentsEndIndex - nCommentsStartIndex > 1) {
                        // In this case the command parameter should be like
                        // AE_Remark :AEServer /C"user input comments"
                        nCommentsStartIndex = nCommentsStartIndex + 1;
                    }
                }
                retVar = strParameter.substr(nCommentsStartIndex, nCommentsEndIndex - nCommentsStartIndex);
                return retVar;
            };
            /// <summary>
            /// Get the alarm tag name in the parameter string for AE Remark
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.GetAlarmTagName = function (parameter) {
                var nStartIndex = this.FindIndexAfterAlarmSourceServer(parameter);
                if (nStartIndex == -1) {
                    // Parameter invalid
                    return null;
                }
                var nPreLength = nStartIndex;
                var strParameter = parameter.substr(nStartIndex);
                var retVar = null;
                var nTagNameStartIndex = this.FindTagNameStartIndex(strParameter);
                if (nTagNameStartIndex == -1 || nTagNameStartIndex == strParameter.length) {
                    return retVar;
                }
                var nTagNameEndIndex = this.FindTagNameEndIndex(strParameter);
                if (nTagNameStartIndex == nTagNameEndIndex) {
                    return retVar;
                }
                if (this.IsQuotationMark(strParameter, nTagNameStartIndex)) {
                    if (nTagNameEndIndex - nTagNameStartIndex > 1) {
                        // In this case the command parameter should be like
                        // AE_Remark :AEServer /T"{Tag Name}"
                        nTagNameStartIndex = nTagNameStartIndex + 1;
                    }
                }
                retVar = strParameter.substr(nTagNameStartIndex, nTagNameEndIndex - nTagNameStartIndex);
                return retVar;
            };
            /// <summary>
            /// Get the alarm tag key in the parameter string for AE Remark
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.GetAlarmTagKey = function (parameter) {
                var nStartIndex = this.FindIndexAfterAlarmSourceServer(parameter);
                if (nStartIndex == -1) {
                    // Parameter invalid
                    return null;
                }
                var nPreLength = nStartIndex;
                var strParameter = parameter.substr(nStartIndex);
                var retVar = null;
                var nTagKeyStartIndex = this.FindTagKeyStartIndex(strParameter);
                if (nTagKeyStartIndex == -1 || nTagKeyStartIndex == strParameter.length) {
                    return retVar;
                }
                var nTagKeyEndIndex = this.FindTagKeyEndIndex(strParameter);
                if (nTagKeyStartIndex == nTagKeyEndIndex) {
                    return retVar;
                }
                retVar = strParameter.substr(nTagKeyStartIndex, nTagKeyEndIndex - nTagKeyStartIndex);
                return retVar;
            };
            /// <summary>
            /// Decode the command parameter.
            /// Leave this method here in case it should expand its function in future
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.DecodeParameter = function (parameter) {
                return decodeURIComponent(parameter);
            };
            /// <summary>
            /// Method to remove '/K***' in AE remark tag connection
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.RemoveAERemarkTagConnection = function (parameter) {
                // According to current design, '/K***' must be the last part of the command string
                if (parameter.IndexOf(this.AE_PARAMETER_SIGN_TAGKEY) != -1) {
                    return parameter.substr(0, parameter.IndexOf(this.AE_PARAMETER_SIGN_TAGKEY)).trim();
                }
                return parameter;
            };
            /// <summary>
            /// Method to find the index after the first part (should be either AlarmSource or AlarmServer) of the parameter
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.FindIndexAfterAlarmSourceServer = function (parameter) {
                var retVar = -1;
                if (this.IsQuotationMark(parameter, 0)) {
                    // Alarm Source or Alarm Server contains space
                    retVar = 1;
                    parameter = parameter.substr(1);
                    var nNextQuotationMarkIndex = parameter.IndexOf(this.AE_PARAMETER_SIGN_QUOTATIONMARK);
                    retVar = retVar + nNextQuotationMarkIndex;
                }
                else {
                    // If the alarm source or alarm server is not quoted then it should contain only one alarm without whitespace
                    var spaceIndex = parameter.IndexOf(this.AE_PARAMETER_SIGN_SPACESLASH);
                    if (spaceIndex > 0) {
                        retVar = spaceIndex;
                    }
                    else {
                        // Parameter invalid
                        retVar = -1;
                    }
                }
                return retVar;
            };
            /// <summary>
            /// Find the start index for the AE command comments.
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.FindCommentsStartIndex = function (parameter) {
                var retVar = -1;
                var nCommentsStartIndex = this.FindIndexAfterSign(parameter.toUpperCase(), this.AE_PARAMETER_SIGN_COMMENTS);
                if (nCommentsStartIndex == -1) {
                    return -1;
                }
                retVar = nCommentsStartIndex;
                if (this.IsParameterEnd(parameter, nCommentsStartIndex) || this.IsSpace(parameter, nCommentsStartIndex)) {
                    // In this case the command parameter should be like
                    // AE_Remark :AEServer /C
                    // or AE_Remark :AEServer /C /P
                    // No pre-input comments
                    return -1;
                }
                return retVar;
            };
            /// <summary>
            /// Find the end index for the AE command comments.
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.FindCommentsEndIndex = function (parameter) {
                var nCommentsStartIndex = this.FindCommentsStartIndex(parameter);
                var retVar = nCommentsStartIndex;
                if (nCommentsStartIndex == -1) {
                    return retVar;
                }
                if (this.IsQuotationMark(parameter, nCommentsStartIndex)) {
                    if (this.IsParameterEnd(parameter, nCommentsStartIndex + 1) || this.IsSpace(parameter, nCommentsStartIndex + 1)) {
                        // User pre-input comments could be
                        // AE_Remark :AEServer /C"
                        // AE_Remark :AEServer /C" /TTagName
                        retVar = nCommentsStartIndex + 1;
                    }
                    else {
                        var nPreLength = nCommentsStartIndex + 1;
                        var strTemp = parameter.substr(nCommentsStartIndex + 1);
                        // Next quotation mark will be the end of the comments
                        retVar = strTemp.IndexOf(this.AE_PARAMETER_SIGN_QUOTATIONMARK) + nPreLength;
                    }
                }
                else {
                    var nPreLength = nCommentsStartIndex;
                    var strTemp = parameter.substr(nCommentsStartIndex);
                    if (strTemp.IndexOf(this.AE_PARAMETER_SIGN_SPACE) != -1) {
                        retVar = nPreLength + strTemp.IndexOf(this.AE_PARAMETER_SIGN_SPACE);
                    }
                    else {
                        retVar = parameter.length;
                    }
                }
                return retVar;
            };
            /// <summary>
            /// Find the start index for the AE Remark command tag name.
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.FindTagNameStartIndex = function (parameter) {
                var retVar = -1;
                var nTagNameStartIndex = this.FindIndexAfterSign(parameter.toUpperCase(), this.AE_PARAMETER_SIGN_TAG);
                if (nTagNameStartIndex == -1) {
                    return retVar;
                }
                retVar = nTagNameStartIndex;
                var nCommentsStartIndex = this.FindCommentsStartIndex(parameter);
                if (nCommentsStartIndex == -1) {
                    // No user pre-input comments
                    return retVar;
                }
                else {
                    if (nTagNameStartIndex > nCommentsStartIndex) {
                        // In some cases it would be like
                        // AE_Remark :AEServer /C"some characters with /T" /TTagName
                        var nCommentsEndIndex = this.FindCommentsEndIndex(parameter);
                        var nPreLength = nCommentsEndIndex;
                        var strTemp = parameter.substr(nCommentsEndIndex);
                        nTagNameStartIndex = this.FindIndexAfterSign(strTemp.toUpperCase(), this.AE_PARAMETER_SIGN_TAG);
                        if (nTagNameStartIndex == -1) {
                            // In this case the command parameter would be like
                            // AE_Remark :AEServer /C"some characters with /T"
                            return -1;
                        }
                        retVar = nPreLength + nTagNameStartIndex;
                    }
                }
                return retVar;
            };
            /// <summary>
            /// Find the end index for the AE Remark command tag name.
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.FindTagNameEndIndex = function (parameter) {
                var nTagNameStartIndex = this.FindTagNameStartIndex(parameter);
                var retVar = nTagNameStartIndex;
                if (this.IsParameterEnd(parameter, nTagNameStartIndex) || this.IsSpace(parameter, nTagNameStartIndex) || nTagNameStartIndex == -1) {
                    return retVar;
                }
                var nPreLength = nTagNameStartIndex;
                var strTemp = parameter.substr(nTagNameStartIndex);
                if (this.IsQuotationMark(strTemp, 0)) {
                    // AERemark Tag could be '/T"{TagWith Space}"'
                    nPreLength = nPreLength + 1;
                    strTemp = strTemp.substr(1);
                    var nNextQuotationMarkIndex = strTemp.IndexOf(this.AE_PARAMETER_SIGN_QUOTATIONMARK);
                    if (nNextQuotationMarkIndex == -1) {
                        // Invalid format for the command parameter
                        // Should be something like AE_Remark :AEServer /T"
                        return -1;
                    }
                    retVar = nPreLength + nNextQuotationMarkIndex;
                }
                else {
                    if (strTemp.IndexOf(this.AE_PARAMETER_SIGN_SPACE) != -1) {
                        retVar = nPreLength + strTemp.IndexOf(this.AE_PARAMETER_SIGN_SPACE);
                    }
                    else {
                        retVar = parameter.length;
                    }
                }
                return retVar;
            };
            /// <summary>
            /// Find the start index for the AE Remark command tag key.
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.FindTagKeyStartIndex = function (parameter) {
                var retVar = -1;
                var nTagKeyStartIndex = this.FindIndexAfterSign(parameter, this.AE_PARAMETER_SIGN_TAGKEY);
                if (nTagKeyStartIndex == -1) {
                    return retVar;
                }
                retVar = nTagKeyStartIndex;
                var nCommentsStartIndex = this.FindCommentsStartIndex(parameter);
                if (nCommentsStartIndex == -1) {
                    // No user pre-input comments
                    return retVar;
                }
                else {
                    if (nTagKeyStartIndex > nCommentsStartIndex) {
                        // In some cases it would be like
                        // AE_Remark :AEServer /C"some characters with /K" /KTagKey
                        var nCommentsEndIndex = this.FindCommentsEndIndex(parameter);
                        if (nTagKeyStartIndex > nCommentsEndIndex) {
                            return retVar;
                        }
                        var nPreLength = nCommentsEndIndex;
                        var strTemp = parameter.substr(nCommentsEndIndex);
                        nTagKeyStartIndex = this.FindIndexAfterSign(strTemp.toUpperCase(), this.AE_PARAMETER_SIGN_TAG);
                        if (nTagKeyStartIndex == -1) {
                            // In this case the command parameter would be like
                            // AE_Remark :AEServer /C"some characters with /K"
                            return -1;
                        }
                        retVar = nPreLength + nTagKeyStartIndex;
                    }
                }
                return retVar;
            };
            /// <summary>
            /// Find the end index for the AE Remark command tag key.
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.FindTagKeyEndIndex = function (parameter) {
                var nTagKeyStartIndex = this.FindTagKeyStartIndex(parameter);
                var retVar = nTagKeyStartIndex;
                if (this.IsParameterEnd(parameter, nTagKeyStartIndex) || this.IsSpace(parameter, nTagKeyStartIndex) || nTagKeyStartIndex == -1) {
                    return retVar;
                }
                var nPreLength = nTagKeyStartIndex;
                var strTemp = parameter.substr(nTagKeyStartIndex);
                if (strTemp.IndexOf(this.AE_PARAMETER_SIGN_SPACE) != -1) {
                    retVar = nPreLength + strTemp.IndexOf(this.AE_PARAMETER_SIGN_SPACE);
                }
                else {
                    retVar = parameter.length;
                }
                return retVar;
            };
            /// <summary>
            /// Find the start index for the AE Shelve command duration.
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.FindShelveDurationStartIndex = function (parameter) {
                var retVar = -1;
                var nShelveDurationStartIndex = this.FindIndexAfterSign(parameter.toUpperCase(), this.AE_PARAMETER_SIGN_DURATION);
                if (nShelveDurationStartIndex == -1) {
                    return retVar;
                }
                retVar = nShelveDurationStartIndex;
                var nCommentsStartIndex = this.FindCommentsStartIndex(parameter);
                if (nCommentsStartIndex == -1) {
                    // No user pre-input comments
                    return retVar;
                }
                else {
                    if (nShelveDurationStartIndex > nCommentsStartIndex) {
                        // In some cases it would be like
                        // AE_Remark :AEServer /C"some characters with /D" /D5
                        var nCommentsEndIndex = this.FindCommentsEndIndex(parameter);
                        if (nShelveDurationStartIndex > nCommentsEndIndex) {
                            return retVar;
                        }
                        var nPreLength = nCommentsEndIndex;
                        var strTemp = parameter.substr(nCommentsEndIndex);
                        nShelveDurationStartIndex = this.FindIndexAfterSign(strTemp.toUpperCase(), this.AE_PARAMETER_SIGN_DURATION);
                        if (nShelveDurationStartIndex == -1) {
                            // In this case the command parameter would be like
                            // AE_Remark :AEServer /C"some characters with /D"
                            return -1;
                        }
                        retVar = nPreLength + nShelveDurationStartIndex;
                    }
                }
                return retVar;
            };
            /// <summary>
            /// Find the end index for the AE Shelve command duration.
            /// </summary>
            /// <param name="parameter"></param>
            AlarmCommandUtils.FindShelveDurationEndIndex = function (parameter) {
                var nShelveDurationStartIndex = this.FindShelveDurationStartIndex(parameter);
                var retVar = nShelveDurationStartIndex;
                if (this.IsParameterEnd(parameter, nShelveDurationStartIndex) || this.IsSpace(parameter, nShelveDurationStartIndex) || nShelveDurationStartIndex == -1) {
                    return retVar;
                }
                var nPreLength = nShelveDurationStartIndex;
                var strTemp = parameter.substr(nShelveDurationStartIndex);
                if (strTemp.IndexOf(this.AE_PARAMETER_SIGN_SPACE) != -1) {
                    retVar = nPreLength + strTemp.IndexOf(this.AE_PARAMETER_SIGN_SPACE);
                }
                else {
                    retVar = parameter.length;
                }
                return retVar;
            };
            /// <summary>
            /// If the charactor is quotation mark.
            /// </summary>
            /// <param name="parameter"></param>
            /// <param name="index"></param>
            AlarmCommandUtils.IsQuotationMark = function (parameter, index) {
                var currentChar = parameter.substr(index, 1);
                if (currentChar == this.AE_PARAMETER_SIGN_QUOTATIONMARK) {
                    return true;
                }
                return false;
            };
            /// <summary>
            /// If the charactor is space.
            /// </summary>
            /// <param name="parameter"></param>
            /// <param name="index"></param>
            AlarmCommandUtils.IsSpace = function (parameter, index) {
                var currentChar = parameter.substr(index, 1);
                if (currentChar == this.AE_PARAMETER_SIGN_SPACE) {
                    return true;
                }
                return false;
            };
            /// <summary>
            /// If reach the end of the parameter.
            /// </summary>
            /// <param name="parameter"></param>
            /// <param name="index"></param>
            AlarmCommandUtils.IsParameterEnd = function (parameter, index) {
                if (index == parameter.length) {
                    return true;
                }
                return false;
            };
            /// <summary>
            /// Find the index after the specified sign.
            /// </summary>
            /// <param name="parameter"></param>
            /// <param name="strSign"></param>
            AlarmCommandUtils.FindIndexAfterSign = function (parameter, strSign) {
                var retVar = -1;
                var currentCursorIndex = 0;
                while (currentCursorIndex <= parameter.length - 2) {
                    var currentChars = parameter.substr(currentCursorIndex, 2);
                    if (currentChars == strSign) {
                        retVar = currentCursorIndex + 2;
                        return retVar;
                    }
                    currentCursorIndex++;
                }
                return retVar;
            };
            AlarmCommandUtils.AE_PARAMETER_SIGN_COMMENTS = "/C";
            AlarmCommandUtils.AE_PARAMETER_SIGN_POPUPDIALOG_COMMENTS = "/P";
            AlarmCommandUtils.AE_PARAMETER_SIGN_DURATION = "/D";
            AlarmCommandUtils.AE_PARAMETER_SIGN_QUOTATIONMARK = "\"";
            AlarmCommandUtils.AE_PARAMETER_SIGN_SPACE = " ";
            AlarmCommandUtils.AE_PARAMETER_SIGN_COLON = ":";
            AlarmCommandUtils.AE_PARAMETER_SIGN_TAG = "/T";
            AlarmCommandUtils.AE_PARAMETER_SIGN_TAGKEY = "/K";
            AlarmCommandUtils.AE_PARAMETER_SIGN_SPACESLASH = " /";
            return AlarmCommandUtils;
        }());
        Util.AlarmCommandUtils = AlarmCommandUtils;
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        var DateTimeCommonUtilities = /** @class */ (function () {
            function DateTimeCommonUtilities() {
                /// <summary>
                /// DateTimeCommonUtilities will calculate client and server time difference, for now it will be 
                /// used to fix polling historical issue when client time > server time.
                /// </summary>
                this.LCIDNameMap = [
                    { id: "1123", name: "ps-AF" },
                    { id: "1164", name: "prs-AF" },
                    { id: "1052", name: "sq-AL" },
                    { id: "5121", name: "ar-DZ" },
                    { id: "11274", name: "es-AR" },
                    { id: "1067", name: "hy-AM" },
                    { id: "3081", name: "en-AU" },
                    { id: "3079", name: "de-AT" },
                    { id: "15361", name: "ar-BH" },
                    { id: "2117", name: "bn-BD" },
                    { id: "1069", name: "eu-ES" },
                    { id: "1059", name: "be-BY" },
                    { id: "2060", name: "fr-BE" },
                    { id: "2067", name: "nl-BE" },
                    { id: "10249", name: "en-BZ" },
                    { id: "8202", name: "es-VE" },
                    { id: "1131", name: "quz-BO" },
                    { id: "16394", name: "es-BO" },
                    { id: "1046", name: "pt-BR" },
                    { id: "2110", name: "ms-BN" },
                    { id: "1026", name: "bg-BG" },
                    { id: "1107", name: "km-KH" },
                    { id: "3084", name: "fr-CA" },
                    { id: "4105", name: "en-CA" },
                    { id: "9225", name: "en-029" },
                    { id: "1027", name: "ca-ES" },
                    { id: "1146", name: "arn-CL" },
                    { id: "13322", name: "es-CL" },
                    { id: "9226", name: "es-CO" },
                    { id: "5130", name: "es-CR" },
                    { id: "1050", name: "hr-HR" },
                    { id: "2092", name: "az-Cyrl-AZ" },
                    { id: "7194", name: "sr-Cyrl-BA" },
                    { id: "8218", name: "bs-Cyrl-BA" },
                    { id: "1104", name: "mn-MN" },
                    { id: "12314", name: "sr-Cyrl-ME" },
                    { id: "10266", name: "sr-Cyrl-RS" },
                    { id: "3098", name: "sr-Cyrl-CS" },
                    { id: "1064", name: "tg-Cyrl-TJ" },
                    { id: "2115", name: "uz-Cyrl-UZ" },
                    { id: "1029", name: "cs-CZ" },
                    { id: "1030", name: "da-DK" },
                    { id: "7178", name: "es-DO" },
                    { id: "2155", name: "quz-EC" },
                    { id: "12298", name: "es-EC" },
                    { id: "3073", name: "ar-EG" },
                    { id: "17418", name: "es-SV" },
                    { id: "1061", name: "et-EE" },
                    { id: "1118", name: "am-ET" },
                    { id: "1080", name: "fo-FO" },
                    { id: "1035", name: "fi-FI" },
                    { id: "2077", name: "sv-FI" },
                    { id: "3131", name: "se-FI" },
                    { id: "8251", name: "sms-FI" },
                    { id: "9275", name: "smn-FI" },
                    { id: "1071", name: "mk-MK" },
                    { id: "1036", name: "fr-FR" },
                    { id: "1150", name: "br-FR" },
                    { id: "1154", name: "oc-FR" },
                    { id: "1155", name: "co-FR" },
                    { id: "1156", name: "gsw-FR" },
                    { id: "1110", name: "gl-ES" },
                    { id: "1079", name: "ka-GE" },
                    { id: "1031", name: "de-DE" },
                    { id: "1070", name: "hsb-DE" },
                    { id: "2094", name: "dsb-DE" },
                    { id: "1032", name: "el-GR" },
                    { id: "1135", name: "kl-GL" },
                    { id: "1158", name: "qut-GT" },
                    { id: "4106", name: "es-GT" },
                    { id: "18442", name: "es-HN" },
                    { id: "1038", name: "hu-HU" },
                    { id: "1039", name: "is-IS" },
                    { id: "1081", name: "hi-IN" },
                    { id: "1093", name: "bn-IN" },
                    { id: "1094", name: "pa-IN" },
                    { id: "1095", name: "gu-IN" },
                    { id: "1096", name: "or-IN" },
                    { id: "1097", name: "ta-IN" },
                    { id: "1098", name: "te-IN" },
                    { id: "1099", name: "kn-IN" },
                    { id: "1100", name: "ml-IN" },
                    { id: "1101", name: "as-IN" },
                    { id: "1102", name: "mr-IN" },
                    { id: "1103", name: "sa-IN" },
                    { id: "1111", name: "kok-IN" },
                    { id: "16393", name: "en-IN" },
                    { id: "1057", name: "id-ID" },
                    { id: "1065", name: "fa-IR" },
                    { id: "2049", name: "ar-IQ" },
                    { id: "2108", name: "ga-IE" },
                    { id: "6153", name: "en-IE" },
                    { id: "1056", name: "ur-PK" },
                    { id: "1037", name: "he-IL" },
                    { id: "1040", name: "it-IT" },
                    { id: "8201", name: "en-JM" },
                    { id: "1041", name: "ja-JP" },
                    { id: "11265", name: "ar-JO" },
                    { id: "1087", name: "kk-KZ" },
                    { id: "1089", name: "sw-KE" },
                    { id: "1042", name: "ko-KR" },
                    { id: "13313", name: "ar-KW" },
                    { id: "1088", name: "ky-KG" },
                    { id: "1108", name: "lo-LA" },
                    { id: "2143", name: "tzm-Latn-DZ" },
                    { id: "1068", name: "az-Latn-AZ" },
                    { id: "4122", name: "hr-BA" },
                    { id: "5146", name: "bs-Latn-BA" },
                    { id: "6170", name: "sr-Latn-BA" },
                    { id: "2141", name: "iu-Latn-CA" },
                    { id: "11290", name: "sr-Latn-ME" },
                    { id: "1128", name: "ha-Latn-NG" },
                    { id: "9242", name: "sr-Latn-RS" },
                    { id: "2074", name: "sr-Latn-CS" },
                    { id: "1091", name: "uz-Latn-UZ" },
                    { id: "1062", name: "lv-LV" },
                    { id: "12289", name: "ar-LB" },
                    { id: "4097", name: "ar-LY" },
                    { id: "5127", name: "de-LI" },
                    { id: "1063", name: "lt-LT" },
                    { id: "1134", name: "lb-LU" },
                    { id: "4103", name: "de-LU" },
                    { id: "5132", name: "fr-LU" },
                    { id: "1086", name: "ms-MY" },
                    { id: "17417", name: "en-MY" },
                    { id: "1125", name: "dv-MV" },
                    { id: "1082", name: "mt-MT" },
                    { id: "2058", name: "es-MX" },
                    { id: "1148", name: "moh-CA" },
                    { id: "6156", name: "fr-MC" },
                    { id: "6145", name: "ar-MA" },
                    { id: "1121", name: "ne-NP" },
                    { id: "1043", name: "nl-NL" },
                    { id: "1122", name: "fy-NL" },
                    { id: "1153", name: "mi-NZ" },
                    { id: "5129", name: "en-NZ" },
                    { id: "19466", name: "es-NI" },
                    { id: "1130", name: "yo-NG" },
                    { id: "1136", name: "ig-NG" },
                    { id: "1044", name: "nb-NO" },
                    { id: "1083", name: "se-NO" },
                    { id: "2068", name: "nn-NO" },
                    { id: "4155", name: "smj-NO" },
                    { id: "6203", name: "sma-NO" },
                    { id: "8193", name: "ar-OM" },
                    { id: "6154", name: "es-PA" },
                    { id: "15370", name: "es-PY" },
                    { id: "3179", name: "quz-PE" },
                    { id: "10250", name: "es-PE" },
                    { id: "1124", name: "fil-PH" },
                    { id: "1045", name: "pl-PL" },
                    { id: "2070", name: "pt-PT" },
                    { id: "1105", name: "bo-CN" },
                    { id: "1144", name: "ii-CN" },
                    { id: "1152", name: "ug-CN" },
                    { id: "20490", name: "es-PR" },
                    { id: "16385", name: "ar-QA" },
                    { id: "13321", name: "en-PH" },
                    { id: "1048", name: "ro-RO" },
                    { id: "1049", name: "ru-RU" },
                    { id: "1092", name: "tt-RU" },
                    { id: "1133", name: "ba-RU" },
                    { id: "1157", name: "sah-RU" },
                    { id: "1159", name: "rw-RW" },
                    { id: "1025", name: "ar-SA" },
                    { id: "1160", name: "wo-SN" },
                    { id: "2052", name: "zh-CN" },
                    { id: "4100", name: "zh-SG" },
                    { id: "18441", name: "en-SG" },
                    { id: "1051", name: "sk-SK" },
                    { id: "1060", name: "sl-SI" },
                    { id: "1074", name: "tn-ZA" },
                    { id: "1076", name: "xh-ZA" },
                    { id: "1077", name: "zu-ZA" },
                    { id: "1078", name: "af-ZA" },
                    { id: "1132", name: "nso-ZA" },
                    { id: "7177", name: "en-ZA" },
                    { id: "3082", name: "es-ES" },
                    { id: "1115", name: "si-LK" },
                    { id: "1053", name: "sv-SE" },
                    { id: "2107", name: "se-SE" },
                    { id: "5179", name: "smj-SE" },
                    { id: "7227", name: "sma-SE" },
                    { id: "1047", name: "rm-CH" },
                    { id: "2055", name: "de-CH" },
                    { id: "2064", name: "it-CH" },
                    { id: "4108", name: "fr-CH" },
                    { id: "1117", name: "iu-Cans-CA" },
                    { id: "1114", name: "syr-SY" },
                    { id: "10241", name: "ar-SY" },
                    { id: "1054", name: "th-TH" },
                    { id: "2128", name: "mn-Mong-CN" },
                    { id: "3076", name: "zh-HK" },
                    { id: "5124", name: "zh-MO" },
                    { id: "1028", name: "zh-TW" },
                    { id: "11273", name: "en-TT" },
                    { id: "7169", name: "ar-TN" },
                    { id: "1055", name: "tr-TR" },
                    { id: "1090", name: "tk-TM" },
                    { id: "14337", name: "ar-AE" },
                    { id: "1058", name: "uk-UA" },
                    { id: "1106", name: "cy-GB" },
                    { id: "1169", name: "gd-GB" },
                    { id: "2057", name: "en-GB" },
                    { id: "1033", name: "en-US" },
                    { id: "21514", name: "es-US" },
                    { id: "14346", name: "es-UY" },
                    { id: "1066", name: "vi-VN" },
                    { id: "9217", name: "ar-YE" },
                    { id: "12297", name: "en-ZW" }
                ];
            }
            DateTimeCommonUtilities.prototype.CalculatelClientServerTimeDiff = function (serverGMT) {
                var diff = 0;
                try {
                    if (serverGMT) {
                        var currentClientTime = Date.now();
                        var serverMS = this.GMTDateParseToMS(serverGMT);
                        // 1 seconds fault-tolerant
                        diff = currentClientTime - serverMS + 1000;
                    }
                }
                catch (_e) {
                    console.log("[CalculatelClientServerTimeDiff] Error>>" + _e.toString());
                }
                finally {
                    return diff;
                }
            };
            DateTimeCommonUtilities.prototype.ConvertLCIDToLCName = function (lcid) {
                for (var i = 0; i < this.LCIDNameMap.length; i++) {
                    if (this.LCIDNameMap[i].id == lcid) {
                        return this.LCIDNameMap[i].name;
                    }
                }
                return "";
            };
            DateTimeCommonUtilities.prototype.ConvertLCNameToLCID = function (name) {
                for (var i = 0; i < this.LCIDNameMap.length; i++) {
                    if (this.LCIDNameMap[i].name == name) {
                        return this.LCIDNameMap[i].id;
                    }
                }
                return "";
            };
            DateTimeCommonUtilities.prototype.GMTDateParseToMS = function (GMTStr) {
                var monthArr = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
                var monthMap = [];
                for (var i = 0; i < 12; i++) {
                    monthMap[monthArr[i]] = i;
                }
                var dat = GMTStr.split(" ");
                var day = Number(dat[1]);
                var month = Number(monthMap[dat[2]]);
                var year = Number(dat[3]);
                var time = dat[4].split(":");
                var hours = Number(time[0]);
                var minutes = Number(time[1]);
                var seconds = Number(time[2]);
                return Date.UTC(year, month, day, hours, minutes, seconds, 500);
            };
            return DateTimeCommonUtilities;
        }());
        Util.DateTimeCommonUtilities = DateTimeCommonUtilities;
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        /// <summary>
        /// HRESULTs we are interested in.
        /// </summary>
        var HRESULT = /** @class */ (function () {
            // Private constructor to prevent this class from being instantiated
            function HRESULT() {
            }
            /// <summary>
            ///     Is this a failure HRESULT?
            ///     NOTE: Any HRESULT other than S_OK is flagged as a failure.
            /// </summary>
            /// 
            /// <param name="hr">
            ///     HRESULT.
            /// </param>
            /// 
            /// <returns>
            ///     Returns "true" if its a failure, "false" otherwise,
            /// </returns>
            HRESULT.FAILED = function (hr) {
                return !HRESULT.SUCCEEDED(hr);
            };
            /// <summary>
            ///     Is this a success HRESULT?
            ///     NOTE: Only the HRESULT S_OK is flagged as a success.
            /// </summary>
            /// 
            /// <param name="hr">
            ///     HRESULT.
            /// </param>
            /// 
            /// <returns>
            ///     Returns "true" if its a success, "false" otherwise,
            /// </returns>
            HRESULT.SUCCEEDED = function (hr) {
                return (hr >= 0);
            };
            /// <summary>
            ///     Format the HRESULT as a string that we can display.
            /// </summary>
            /// 
            /// <param name="hr">
            ///     HRESULT to format.
            /// </param>
            /// 
            /// <returns>
            ///     Formatted HRESULT (e.g., "0x80046F5A").
            /// </returns>
            HRESULT.ToString = function (hr) {
                return "0x" + (hr < 0 ? (0xFFFFFFFF + hr + 1).toString(16) : hr.toString(16)).toUpperCase();
            };
            /// <summary>
            ///     HRESULT S_OK returned by the LDPoolGateway web service.
            /// </summary>
            HRESULT.S_OK = 0;
            /// <summary>
            ///     The specified Operation Instance ID was not found on the
            ///     FactoryTalk ViewPoint Live Data Pool Gateway server.
            /// </summary>
            HRESULT.LDP_S_OPERATION_INSTANCE_ID_NOT_FOUND = 307201;
            /// <summary>
            ///     The Data Item write operation for the specified Operation Instance ID 
            ///     has not yet completed on the FactoryTalk ViewPoint Live Data Pool Gateway server.
            /// </summary>
            HRESULT.LDP_S_OPERATION_INSTANCE_ID_WRITE_PENDING = 307202;
            return HRESULT;
        }());
        Util.HRESULT = HRESULT;
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Client;
    (function (Client) {
        var Util;
        (function (Util) {
            var VPAdminWebService;
            (function (VPAdminWebService) {
                var ViewPointAdminWebServiceSoapClient = /** @class */ (function () {
                    function ViewPointAdminWebServiceSoapClient(m_host, m_vDir, m_timeout) {
                        this.m_timeout = 10;
                        this.m_timeout = m_timeout;
                        this.WebProtocol = window.location.protocol;
                        this.endPointAddress = System.String.Format(CultureInfo.InvariantCulture, "{0}/{1}VPAdminServer/Service.asmx", m_host, m_vDir);
                        this.GetRunningViewApplicationCompleted = new Events.Delegate();
                    }
                    ViewPointAdminWebServiceSoapClient.prototype.GetRunningViewApplicationAsync = function (clientIdentifier, userState) {
                        var data = "<GetRunningViewApplication xmlns=\"http://tempuri.org/\">";
                        data += "<clientIdentifier><ExtensionData /> <UniqueClientIdentifier>" + clientIdentifier.UniqueClientIdentifier + "</UniqueClientIdentifier><UserHost>" + clientIdentifier.UserHost + "</UserHost><UserName>" + clientIdentifier.UserName + "</UserName></clientIdentifier> ";
                        data += "</GetRunningViewApplication></soap:Body></soap:Envelope> ";
                        WebServiceUtils.CallWebService(data, this.endPointAddress, "http://tempuri.org/GetRunningViewApplication", "GetRunningViewApplicationResponse", this.GetRunningViewApplicationCompleted, "GetRunningViewApplicationResponse", userState);
                    };
                    return ViewPointAdminWebServiceSoapClient;
                }());
                VPAdminWebService.ViewPointAdminWebServiceSoapClient = ViewPointAdminWebServiceSoapClient;
                var ClientIdentifier = /** @class */ (function () {
                    function ClientIdentifier() {
                        this.UniqueClientIdentifierField = null;
                        this.UserHostField = null;
                        this.UserNameField = null;
                    }
                    Object.defineProperty(ClientIdentifier.prototype, "UniqueClientIdentifier", {
                        get: function () {
                            return this.UniqueClientIdentifierField;
                        },
                        set: function (value) {
                            if ((this.UniqueClientIdentifierField === value != true)) {
                                this.UniqueClientIdentifierField = value;
                            }
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(ClientIdentifier.prototype, "UserHost", {
                        get: function () {
                            return this.UserHostField;
                        },
                        set: function (value) {
                            if ((this.UserHostField === value != true)) {
                                this.UserHostField = value;
                            }
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(ClientIdentifier.prototype, "UserName", {
                        get: function () {
                            return this.UserNameField;
                        },
                        set: function (value) {
                            if ((this.UserNameField === value != true)) {
                                this.UserNameField = value;
                            }
                        },
                        enumerable: true,
                        configurable: true
                    });
                    return ClientIdentifier;
                }());
                VPAdminWebService.ClientIdentifier = ClientIdentifier;
                var ConfigServiceStatusCode;
                (function (ConfigServiceStatusCode) {
                    ConfigServiceStatusCode[ConfigServiceStatusCode["Success"] = 0] = "Success";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["UnableToLoadApplication"] = 1] = "UnableToLoadApplication";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["UnknownScopeValue"] = 2] = "UnknownScopeValue";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["UnknownApplicationName"] = 3] = "UnknownApplicationName";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["ApplicationHasNotLoaded"] = 4] = "ApplicationHasNotLoaded";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["UnableToOpenApplication"] = 5] = "UnableToOpenApplication";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["DisplaysHaveNotLoaded"] = 6] = "DisplaysHaveNotLoaded";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["InitialDisplayNotProvided"] = 7] = "InitialDisplayNotProvided";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["PublishNotStart"] = 8] = "PublishNotStart";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["PublishInProgress"] = 9] = "PublishInProgress";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["PublishErrorProgressStop"] = 10] = "PublishErrorProgressStop";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["PublishErrorProgressContinue"] = 11] = "PublishErrorProgressContinue";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["PublishCompleted"] = 12] = "PublishCompleted";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["EventListNotAvailable"] = 13] = "EventListNotAvailable";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["WindowsServiceNotAvailable"] = 14] = "WindowsServiceNotAvailable";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["WebServiceNotAvailable"] = 15] = "WebServiceNotAvailable";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["SessionIsCurrentlyInUseByAnotherClient"] = 16] = "SessionIsCurrentlyInUseByAnotherClient";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["SessionIsNotInUse"] = 17] = "SessionIsNotInUse";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["SessionIsAlreadyActive"] = 18] = "SessionIsAlreadyActive";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["SessionHasBeenRenewed"] = 19] = "SessionHasBeenRenewed";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["BackgroundUpdateCouldNotBeCancelled"] = 20] = "BackgroundUpdateCouldNotBeCancelled";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["BackgroundUpdateCurrentlyInProgress"] = 21] = "BackgroundUpdateCurrentlyInProgress";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["LicenseSessionOrClientIDIsNullOrEmpty"] = 22] = "LicenseSessionOrClientIDIsNullOrEmpty";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["LicenseAllInUse"] = 23] = "LicenseAllInUse";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["LicenseAlreadyAssignedForThisSessionAndClientID"] = 24] = "LicenseAlreadyAssignedForThisSessionAndClientID";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["NoRunningViewApplication"] = 25] = "NoRunningViewApplication";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["ClientIPAddressNotAvailable"] = 26] = "ClientIPAddressNotAvailable";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["NoLicensesAllocatedForThisClientComputer"] = 27] = "NoLicensesAllocatedForThisClientComputer";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["WebServerProtectionFailed"] = 28] = "WebServerProtectionFailed";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["ApplicationNotRunning"] = 29] = "ApplicationNotRunning";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["LoginRequired"] = 30] = "LoginRequired";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["NotImplemented"] = 31] = "NotImplemented";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["NoManifestForApplication"] = 32] = "NoManifestForApplication";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["SecuredObjectNotFound"] = 33] = "SecuredObjectNotFound";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["NotAuthorized"] = 34] = "NotAuthorized";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["SessionHasExpired"] = 35] = "SessionHasExpired";
                    ConfigServiceStatusCode[ConfigServiceStatusCode["SessionNotFound"] = 36] = "SessionNotFound";
                })(ConfigServiceStatusCode = VPAdminWebService.ConfigServiceStatusCode || (VPAdminWebService.ConfigServiceStatusCode = {}));
            })(VPAdminWebService = Util.VPAdminWebService || (Util.VPAdminWebService = {}));
        })(Util = Client.Util || (Client.Util = {}));
    })(Client = ViewPoint.Client || (ViewPoint.Client = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        var RNADiagSeverity;
        (function (RNADiagSeverity) {
            RNADiagSeverity[RNADiagSeverity["RNADiagSeverity_Error"] = 1] = "RNADiagSeverity_Error";
            RNADiagSeverity[RNADiagSeverity["RNADiagSeverity_Warning"] = 2] = "RNADiagSeverity_Warning";
            RNADiagSeverity[RNADiagSeverity["RNADiagSeverity_Info"] = 4] = "RNADiagSeverity_Info";
            RNADiagSeverity[RNADiagSeverity["RNADiagSeverity_Audit"] = 8] = "RNADiagSeverity_Audit";
        })(RNADiagSeverity = Util.RNADiagSeverity || (Util.RNADiagSeverity = {}));
        /// <summary>
        ///     Trace and Diagnostics helper.
        /// </summary>
        var TraceAndDiagnosticsHelper = /** @class */ (function () {
            function TraceAndDiagnosticsHelper() {
            }
            /// <summary>
            ///     Log the specified message to both the Trace and the FactoryTalk Diagnostics log.
            ///     NOTE: For the Silverlight implementation, we currently simply log a Trace message.
            /// </summary>
            /// 
            /// <param name="strMessage">
            ///     Message to log.
            /// </param>
            /// 
            /// <param name="severity">
            ///     Severity to log the message with FactoryTalk Diagnostics.
            /// </param>
            TraceAndDiagnosticsHelper.LogMessage = function (strMessage, severity) {
                // Sanity check...
                console.assert(!System.String.IsNullOrEmpty(strMessage));
                // Trace log...
                Util.Trace.WriteLine(strMessage);
            };
            /// <summary>
            ///     Log the specified message to the Trace log.
            /// </summary>
            /// 
            /// <param name="strMessage">
            ///     Message to log.
            /// </param>
            TraceAndDiagnosticsHelper.LogMessageToTraceLog = function (strMessage) {
                // Sanity check...
                console.assert(!System.String.IsNullOrEmpty(strMessage));
                // Trace log...
                Util.Trace.WriteLine(strMessage);
            };
            /// <summary>
            ///     Log the specified message to both the FactoryTalk Diagnostics log.
            ///     NOTE: For the Silverlight implementation, we currently simply log a Trace message.
            /// </summary>
            /// 
            /// <param name="strMessage">
            ///     Message to log.
            /// </param>
            /// 
            /// <param name="severity">
            ///     Severity to log the message with FactoryTalk Diagnostics.
            /// </param>
            TraceAndDiagnosticsHelper.LogMessageToDiagnosticsLog = function (strMessage, severity) {
                // Sanity check...
                console.assert(!System.String.IsNullOrEmpty(strMessage));
                // Trace log...
                Util.Trace.WriteLine(strMessage);
            };
            return TraceAndDiagnosticsHelper;
        }());
        Util.TraceAndDiagnosticsHelper = TraceAndDiagnosticsHelper;
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Client;
    (function (Client) {
        var Util;
        (function (Util) {
            var LDWebService;
            (function (LDWebService) {
                var RocketLiveDataServiceSoapClient = /** @class */ (function () {
                    function RocketLiveDataServiceSoapClient(m_host, m_vDir) {
                        this.m_timeout = 10;
                        this._deferreds = new Array();
                        this._deferredsCallBackInfo = new Array();
                        this._callStack = new Array();
                        this.WebProtocol = window.location.protocol;
                        this.GetDataExCompleted = new Events.Delegate();
                        this.WriteValuesCompleted = new Events.Delegate();
                        this.GetWriteCompletionStatusCompleted = new Events.Delegate();
                        this.DefineCollectionCompleted = new Events.Delegate();
                        this.AddDataItemsToCollectionExCompleted = new Events.Delegate();
                        this.AddDataItemsToCollectionCompleted = new Events.Delegate();
                        this.endPointAddress = System.String.Format(CultureInfo.InvariantCulture, "{0}/{1}LDServer/Service.asmx", m_host, m_vDir);
                    }
                    Object.defineProperty(RocketLiveDataServiceSoapClient.prototype, "Deferreds", {
                        get: function () {
                            return this._deferreds;
                        },
                        set: function (value) {
                            this._deferreds = value;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(RocketLiveDataServiceSoapClient.prototype, "DeferredsCallBackInfo", {
                        get: function () {
                            return this._deferredsCallBackInfo;
                        },
                        set: function (value) {
                            this._deferredsCallBackInfo = value;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(RocketLiveDataServiceSoapClient.prototype, "CallStack", {
                        get: function () {
                            return this._callStack;
                        },
                        set: function (value) {
                            this._callStack = value;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    RocketLiveDataServiceSoapClient.prototype.GetDataExAsync_1 = function () {
                        if (this.CallStack.length >= 1) {
                            var that = this;
                            for (var i = 0; i < this.CallStack.length; i++) {
                                var obj = this.CallStack[i];
                                this.GetDataExAsync(obj.DispName, obj.Id, obj.parameters, obj.properties, obj.display);
                            }
                            for (var i = 0; i < this.Deferreds.length; i++) {
                                $.when(null, this.Deferreds[i]).always(function () {
                                    if (that.DeferredsCallBackInfo.length == 0) {
                                        console.log('GetDataEx error is not handled correctly');
                                        that.GetDataExCompleted.raise(null, XMLUtils.XML2.createWebServiceCallbackEventArgs(null, null, "GetDataEx error is not handled correctly", obj.display, null));
                                    }
                                    for (var i = 0; i < that.DeferredsCallBackInfo.length; i++) {
                                        try {
                                            that.GetDataExCompleted.raise(null, that.DeferredsCallBackInfo[i]);
                                        }
                                        catch (e) {
                                            break;
                                        }
                                    }
                                });
                            }
                            this.DeferredsCallBackInfo = new Array();
                            this.CallStack = new Array();
                            this.Deferreds = new Array();
                        }
                    };
                    RocketLiveDataServiceSoapClient.prototype.GetDataExAsync_LDPoolUpdateManager = function (displayPath, displayId, strParameters, bGetProperties, userState) {
                        var paramXML = "<GetDataEx xmlns=\"http://tempuri.org/\">";
                        paramXML += "<displayPath>" + displayPath + "</displayPath>";
                        paramXML += "<displayId>" + displayId + "</displayId>";
                        paramXML += "<strParameters>" + strParameters + "</strParameters>";
                        paramXML += "<bGetProperties>" + bGetProperties + "</bGetProperties>";
                        paramXML += "</GetDataEx></soap:Body></soap:Envelope>";
                        WebServiceUtils.CallWebServiceWithTimeout(paramXML, this.endPointAddress, "http://tempuri.org/GetDataEx", "GetDataExResult", this.GetDataExCompleted, "GetDataEx", userState, this.m_timeout);
                    };
                    RocketLiveDataServiceSoapClient.prototype.GetDataExAsync = function (displayPath, displayId, strParameters, bGetProperties, userState) {
                        var paramXML = "<GetDataEx xmlns=\"http://tempuri.org/\">";
                        if (!System.String.IsNullOrEmpty(displayPath)) {
                            paramXML += "<displayPath>" + displayPath + "</displayPath>";
                        }
                        paramXML += "<displayId>" + displayId + "</displayId>";
                        paramXML += "<strParameters>" + strParameters + "</strParameters>";
                        paramXML += "<bGetProperties>" + bGetProperties + "</bGetProperties>";
                        paramXML += "</GetDataEx></soap:Body></soap:Envelope>";
                        //WebServiceUtils.CallWebService(paramXML, this.endPointAddress, "http://tempuri.org/GetDataEx", "GetDataExResult", this.GetDataExCompleted, "GetDataEx", userState);
                        if (WebServiceUtils.SoapHeader.Header != undefined)
                            paramXML = WebServiceUtils.SoapHeader.Header + paramXML;
                        else
                            paramXML = WebServiceUtils.GetNormalHeader() + paramXML;
                        var that = this;
                        this.Deferreds.push($.ajax({
                            url: this.endPointAddress,
                            type: "POST",
                            dataType: "xml",
                            data: paramXML,
                            contentType: "text/xml; charset=utf-8",
                            processData: false,
                            headers: { SOAPAction: "http://tempuri.org/GetDataEx" },
                            timeout: this.m_timeout * 1000,
                            error: function (data, textStatus, xhr) {
                                that.DeferredsCallBackInfo.push(XMLUtils.XML2.createWebServiceCallbackEventArgs(null, null, xhr, userState, null));
                            },
                            success: function (data, textStatus, xhr) {
                                var result = null;
                                try {
                                    result = XMLUtils.XML2.XML2jsobj($(xhr.responseXML).find("GetDataExResult")[0]);
                                }
                                catch (e) {
                                    result = XMLUtils.XML2.XML2jsobj($(xhr.responseXML).find("GetDataEx" + "Response")[0]);
                                }
                                that.DeferredsCallBackInfo.push(XMLUtils.XML2.createWebServiceCallbackEventArgs(result, null, null, userState, null));
                            }
                        }));
                    };
                    RocketLiveDataServiceSoapClient.prototype.WriteValuesAsync = function (displayPath, displayId, strParameters, writeValueRequestInformationList, userState) {
                        var paramXML = "<WriteValues xmlns=\"http://tempuri.org/\" xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\">";
                        paramXML += " <displayPath>" + displayPath + "</displayPath>";
                        paramXML += " <displayId>" + displayId + "</displayId>";
                        paramXML += " <strParameters>" + strParameters + "</strParameters>";
                        paramXML += "<writeValueRequestInformationList>";
                        for (var i = 0; i < writeValueRequestInformationList.length; i++) {
                            paramXML += "<WRI>";
                            paramXML += "<ObjectReference>";
                            paramXML += "<IID>" + writeValueRequestInformationList[i].ObjectReference.IID + "</IID>";
                            paramXML += "<PID>" + writeValueRequestInformationList[i].ObjectReference.PID + "</PID>";
                            paramXML += "</ObjectReference>";
                            paramXML += "<OperationInstanceID>" + writeValueRequestInformationList[i].OperationInstanceID + "</OperationInstanceID>";
                            if (typeof writeValueRequestInformationList[i].Value == "number") {
                                if (writeValueRequestInformationList[i].Value >= System.Int32.MinValue && writeValueRequestInformationList[i].Value <= System.Int32.MaxValue && writeValueRequestInformationList[i].Value.toString().indexOf(System.Globalization.CultureInfo.InvariantCulture.NumberFormat.NumberDecimalSeparator) < 0 && writeValueRequestInformationList[i].Value.toString().toLowerCase().indexOf("e") < 0)
                                    paramXML += "<Value xmlns:d6p1=\"http://www.w3.org/2001/XMLSchema\" i:type=\"d6p1:int\">" + writeValueRequestInformationList[i].Value + "</Value>";
                                else
                                    paramXML += "<Value xmlns:d6p1=\"http://www.w3.org/2001/XMLSchema\" i:type=\"d6p1:double\">" + writeValueRequestInformationList[i].Value + "</Value>";
                            }
                            else
                                paramXML += "<Value xmlns:d6p1=\"http://www.w3.org/2001/XMLSchema\" i:type=\"d6p1:string\">" + System.String.Decode(writeValueRequestInformationList[i].Value) + "</Value>";
                            paramXML += "</WRI>";
                        }
                        paramXML += "</writeValueRequestInformationList>";
                        paramXML += "</WriteValues></soap:Body></soap:Envelope> ";
                        WebServiceUtils.CallWebService(paramXML, this.endPointAddress, "http://tempuri.org/WriteValues", "operationStatus", this.WriteValuesCompleted, "WriteValues", userState);
                    };
                    RocketLiveDataServiceSoapClient.prototype.GetWriteCompletionStatusAsync = function (operationInstantIDList) {
                        var paramXML = "<GetWriteCompletionStatus  xmlns=\"http://tempuri.org/\">";
                        paramXML += "<operationInstantIDList>";
                        for (var i = 0; i < operationInstantIDList.length; i++)
                            paramXML += "<string>" + operationInstantIDList[i] + "</string>";
                        paramXML += "</operationInstantIDList>";
                        paramXML += "</GetWriteCompletionStatus ></soap:Body></soap:Envelope> ";
                        WebServiceUtils.CallWebService(paramXML, this.endPointAddress, "http://tempuri.org/GetWriteCompletionStatus", "writeValueStatusList", this.GetWriteCompletionStatusCompleted, "GetWriteCompletionStatus", null);
                    };
                    RocketLiveDataServiceSoapClient.prototype.DefineCollection = function (bstrCollectionName, bstrParameters, contents) {
                        var paramXML = "<DefineCollection xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\"  xmlns=\"http://tempuri.org/\">";
                        paramXML += "<collectionDesc>";
                        paramXML += "<bstrCollectionName>" + bstrCollectionName + "</bstrCollectionName>";
                        paramXML += "<bstrParameters>" + bstrParameters + " </bstrParameters>";
                        paramXML += "</collectionDesc>";
                        paramXML += "<contents xmlns:d4p1=\"http://www.w3.org/2001/XMLSchema\" i:type=\"d4p1:base64Binary\">" + contents + "</contents>";
                        paramXML += "</DefineCollection></soap:Body></soap:Envelope>";
                        WebServiceUtils.CallWebService(paramXML, this.endPointAddress, "http://tempuri.org/DefineCollection", "DefineCollectionResponse", this.DefineCollectionCompleted, "DefineCollection", null);
                    };
                    RocketLiveDataServiceSoapClient.prototype.AddDataItemsToCollectionEx = function (collectionDesc, dataItemsCollection, bNormalize) {
                        var paramXML = "<AddDataItemsToCollectionEx  xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\"  xmlns=\"http://tempuri.org/\">";
                        paramXML += "<collectionDesc>";
                        paramXML += "<bstrCollectionName>" + collectionDesc.bstrCollectionName + "</bstrCollectionName>";
                        paramXML += "<bstrParameters>" + collectionDesc.bstrParameters + "</bstrParameters>";
                        paramXML += "</collectionDesc>";
                        paramXML += "<dataItems>";
                        for (var i = 0; i < dataItemsCollection.Count; i++) {
                            paramXML += "<DataItemDescEx>";
                            paramXML += "<clientData>";
                            paramXML += "<bstrObjectID>" + dataItemsCollection.ElementAt(i).clientData.bstrObjectID + "</bstrObjectID>";
                            if (dataItemsCollection.ElementAt(i).clientData.bstrPropertyName)
                                paramXML += "<bstrPropertyName>" + dataItemsCollection.ElementAt(i).clientData.bstrPropertyName + "</bstrPropertyName>";
                            paramXML += "</clientData>";
                            paramXML += "<bstrItemId>" + dataItemsCollection.ElementAt(i).bstrItemId + "</bstrItemId>";
                            paramXML += "</DataItemDescEx>";
                        }
                        paramXML += "</dataItems>";
                        paramXML += "<bNormalize>" + bNormalize + "</bNormalize>";
                        paramXML += "</AddDataItemsToCollectionEx ></soap:Body></soap:Envelope>";
                        return paramXML;
                        //WebServiceUtils.CallWebService(paramXML, this.endPointAddress, "http://tempuri.org/AddDataItemsToCollectionEx", "AddDataItemsToCollectionExResponse", this.AddDataItemsToCollectionExCompleted, "AddDataItemsToCollectionExResponse", null);
                    };
                    RocketLiveDataServiceSoapClient.prototype.AddDataItemsToCollection = function (collectionDesc, dataItemsCollection, bNormalize) {
                        var paramXML = "<AddDataItemsToCollection xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\"  xmlns=\"http://tempuri.org/\">";
                        paramXML += "<collectionDesc>";
                        paramXML += "<bstrCollectionName>" + collectionDesc.bstrCollectionName + "</bstrCollectionName>";
                        paramXML += "<bstrParameters>" + collectionDesc.bstrParameters + "</bstrParameters>";
                        paramXML += "</collectionDesc>";
                        paramXML += "<dataItems>";
                        for (var i = 0; i < dataItemsCollection.Count; i++) {
                            paramXML += "<DataItemDesc>";
                            paramXML += "<bstrObjectID>" + dataItemsCollection.ElementAt(i).clientData.bstrObjectID + "</bstrObjectID>";
                            paramXML += "<bstrPropertyName>" + dataItemsCollection.ElementAt(i).clientData.bstrPropertyName + "</bstrPropertyName>";
                            paramXML += "</DataItemDesc>";
                        }
                        paramXML += "</dataItems>";
                        paramXML += "</AddDataItemsToCollection></soap:Body></soap:Envelope>";
                        WebServiceUtils.CallWebService(paramXML, this.endPointAddress, "http://tempuri.org/AddDataItemsToCollection", "AddDataItemsToCollectionResponse", this.AddDataItemsToCollectionCompleted, "AddDataItemsToCollectionResponse", null);
                    };
                    return RocketLiveDataServiceSoapClient;
                }());
                LDWebService.RocketLiveDataServiceSoapClient = RocketLiveDataServiceSoapClient;
                var CollectionDesc = /** @class */ (function () {
                    function CollectionDesc() {
                    }
                    Object.defineProperty(CollectionDesc.prototype, "bstrCollectionName", {
                        get: function () {
                            return this.bstrCollectionNameField;
                        },
                        set: function (value) {
                            this.bstrCollectionNameField = value;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(CollectionDesc.prototype, "bstrParameters", {
                        get: function () {
                            return this.bstrParametersField;
                        },
                        set: function (value) {
                            this.bstrParametersField = value;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    return CollectionDesc;
                }());
                LDWebService.CollectionDesc = CollectionDesc;
            })(LDWebService = Util.LDWebService || (Util.LDWebService = {}));
        })(Util = Client.Util || (Client.Util = {}));
    })(Client = ViewPoint.Client || (ViewPoint.Client = {}));
})(ViewPoint || (ViewPoint = {}));
var WebServiceUtils;
(function (WebServiceUtils) {
    var Util = /** @class */ (function () {
        function Util() {
        }
        Util.isTimeout = false;
        Util.isError = false;
        return Util;
    }());
    WebServiceUtils.Util = Util;
    var SoapHeader = /** @class */ (function () {
        function SoapHeader() {
        }
        return SoapHeader;
    }());
    WebServiceUtils.SoapHeader = SoapHeader;
    //Generic method to call a web service asyncroniously
    function CallWebService(xml, address, soapAction, resultName, callbackEventHandler, webServiceName, userState) {
        if (WebServiceUtils.SoapHeader.Header != undefined)
            xml = WebServiceUtils.SoapHeader.Header + xml;
        else
            xml = WebServiceUtils.GetNormalHeader() + xml;
        $.ajax({
            url: address,
            type: "POST",
            dataType: "xml",
            data: xml,
            contentType: "text/xml; charset=utf-8",
            processData: false,
            headers: { SOAPAction: soapAction },
            complete: function (xhr, status) {
                var params = null;
                // pass the server response date to calculate difference time.
                var serverResponseDate = xhr.getResponseHeader('Date');
                switch (status) {
                    case "success":
                        var result = null;
                        try {
                            result = XMLUtils.XML2.XML2jsobj($(xhr.responseXML).find(resultName)[0]);
                        }
                        catch (e) {
                            result = XMLUtils.XML2.XML2jsobj($(xhr.responseXML).find(webServiceName + "Response")[0]);
                        }
                        params = XMLUtils.XML2.createWebServiceCallbackEventArgs(result, null, null, userState, serverResponseDate);
                        callbackEventHandler.raise(null, params);
                        break;
                    case "error":
                        var error = null;
                        try {
                            error = { Message: XMLUtils.XML2.XML2jsobj($(xhr.responseXML.documentElement)[0])["soap:Body"]["soap:Fault"].faultstring };
                        }
                        catch (e) {
                            error = { Message: "Unable to connect to the web service" };
                        }
                        params = XMLUtils.XML2.createWebServiceCallbackEventArgs(null, null, error, userState, serverResponseDate);
                        callbackEventHandler.raise(null, params);
                        break;
                    case "parsererror":
                        var error = null;
                        try {
                            error = { Message: XMLUtils.XML2.XML2jsobj($(xhr.responseXML.documentElement)[0])["soap:Body"]["soap:Fault"].faultstring };
                        }
                        catch (e) {
                            error = { Message: "There were some errors parsing the result" };
                        }
                        params = XMLUtils.XML2.createWebServiceCallbackEventArgs(null, null, error, userState, serverResponseDate);
                        callbackEventHandler.raise(null, params);
                        break;
                }
            },
        });
    }
    WebServiceUtils.CallWebService = CallWebService;
    //Generic method to call a web service asyncroniously
    function CallWebServiceJson(jsonData, jsonAddress, callbackEventHandler) {
        $.ajax({
            url: jsonAddress,
            type: "POST",
            dataType: "json",
            data: jsonData,
            contentType: "application/json; charset=utf-8",
            processData: false,
            complete: function (xhr, status) {
                callbackEventHandler.raise(null, xhr);
            },
        });
    }
    WebServiceUtils.CallWebServiceJson = CallWebServiceJson;
    //Generic method to call a web service asyncroniously
    function CallWebServiceWithTimeout(xml, address, soapAction, resultName, callbackEventHandler, webServiceName, userState, timeout) {
        if (WebServiceUtils.SoapHeader.Header != undefined)
            xml = WebServiceUtils.SoapHeader.Header + xml;
        else
            xml = WebServiceUtils.GetNormalHeader() + xml;
        $.ajax({
            url: address,
            type: "POST",
            dataType: "xml",
            data: xml,
            contentType: "text/xml; charset=utf-8",
            processData: false,
            headers: { SOAPAction: soapAction },
            timeout: timeout * 1000,
            complete: function (xhr, status) {
                var params = null;
                switch (status) {
                    case "success":
                        var result = null;
                        try {
                            result = XMLUtils.XML2.XML2jsobj($(xhr.responseXML).find(resultName)[0]);
                        }
                        catch (e) {
                            result = XMLUtils.XML2.XML2jsobj($(xhr.responseXML).find(webServiceName + "Response")[0]);
                        }
                        WebServiceUtils.Util.isTimeout = false;
                        WebServiceUtils.Util.isError = false;
                        params = XMLUtils.XML2.createWebServiceCallbackEventArgs(result, null, null, userState, null);
                        callbackEventHandler.raise(null, params);
                        break;
                    case "error":
                        var error = null;
                        try {
                            error = { Message: XMLUtils.XML2.XML2jsobj($(xhr.responseXML.documentElement)[0])["soap:Body"]["soap:Fault"].faultstring };
                        }
                        catch (e) {
                            error = { Message: "Unable to connect to the web service" };
                        }
                        params = XMLUtils.XML2.createWebServiceCallbackEventArgs(null, null, error, userState, null);
                        WebServiceUtils.Util.isError = true;
                        callbackEventHandler.raise(null, params);
                        break;
                    case "timeout":
                        params = XMLUtils.XML2.createWebServiceCallbackEventArgs(null, null, "timeout", userState, null);
                        WebServiceUtils.Util.isTimeout = true;
                        callbackEventHandler.raise(null, params);
                        break;
                    case "parsererror":
                        var error = null;
                        try {
                            error = { Message: XMLUtils.XML2.XML2jsobj($(xhr.responseXML.documentElement)[0])["soap:Body"]["soap:Fault"].faultstring };
                        }
                        catch (e) {
                            error = { Message: "There were some errors parsing the result" };
                        }
                        params = XMLUtils.XML2.createWebServiceCallbackEventArgs(null, null, error, userState, null);
                        callbackEventHandler.raise(null, params);
                        break;
                }
            },
        });
    }
    WebServiceUtils.CallWebServiceWithTimeout = CallWebServiceWithTimeout;
    function GetNormalHeader() {
        var header = "<?xml version=\"1.0\" encoding=\"utf-8\"?>";
        header += "<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">";
        header += "<soap:Body>";
        return header;
    }
    WebServiceUtils.GetNormalHeader = GetNormalHeader;
    function GetEncryptedHeader(sessionID) {
        var encryptedsessionID = ViewPoint.Util.ViewPointHelper.EncryptIt(sessionID, 0);
        var header = "<?xml version=\"1.0\" encoding=\"utf-8\"?>";
        header += "<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">";
        header += "<soap:Header><VPHeader xmlns=\"http://www.silverlight.net\">" + encryptedsessionID + "</VPHeader></soap:Header>";
        header += "<soap:Body>";
        return header;
    }
    WebServiceUtils.GetEncryptedHeader = GetEncryptedHeader;
})(WebServiceUtils || (WebServiceUtils = {}));
var XMLUtils;
(function (XMLUtils) {
    var XML2 = /** @class */ (function () {
        function XML2() {
        }
        //Converts XML DOM to "json like" object 
        XML2.XML2jsobj = function (node) {
            var data = {};
            // element attributes
            var c, cn;
            if (node.hasAttributes()) {
                for (c = 0; cn = node.attributes[c]; c++) {
                    this.Add(cn.name, cn.value, data);
                }
            }
            // child elements
            if (node.hasChildNodes()) {
                for (c = 0; cn = node.childNodes[c]; c++) {
                    if (cn.nodeType == 1) {
                        if (CurrentBrowser.browserName() == "Firefox" && (cn.nodeName == "GetUpdatesResult" || cn.nodeName == "strDatasets")) {
                            this.Add(cn.nodeName, cn.textContent, data);
                        }
                        else {
                            if (cn.childNodes.length == 1 && cn.firstChild.nodeType == 3) {
                                // text value
                                this.Add(cn.nodeName, cn.firstChild.nodeValue, data, cn);
                            }
                            else if (cn.childNodes.length == 1 && cn.firstChild.nodeType == 4) {
                                // text value
                                this.Add(cn.nodeName, cn.firstChild.wholeText, data, cn);
                            }
                            else {
                                // sub-object
                                this.Add(cn.nodeName, XML2.XML2jsobj(cn), data);
                            }
                        }
                    }
                }
            }
            return data;
        };
        // append a value
        XML2.Add = function (name, value, data, cn) {
            if (name == "string")
                name = "Text";
            if (data[name]) {
                if (data[name].constructor != Array) {
                    data[name] = [data[name]];
                }
                data[name][data[name].length] = value;
            }
            else {
                data[name] = value;
                if (cn && name == "V" && cn.attributes[0]) {
                    try {
                        var type = cn.attributes[0].value.replace("xsd:", "");
                        data["V_Type"] = type;
                    }
                    catch (e) {
                        data["V_Type"] = "";
                    }
                }
            }
        };
        //Creates an object that interacts with the callback functions.
        XML2.createWebServiceCallbackEventArgs = function (result, cancelled, error, userState, responseDate) {
            var data = {};
            data["Result"] = result;
            data["Cancelled"] = cancelled;
            data["Error"] = error;
            data["UserState"] = userState;
            data["ResponseDate"] = responseDate;
            return data;
        };
        return XML2;
    }());
    XMLUtils.XML2 = XML2;
})(XMLUtils || (XMLUtils = {}));
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        /// <summary>
        ///     String Service local proxy.
        /// </summary>
        var OperationQueue = /** @class */ (function () {
            //region Initialization and cleanup.
            /// <summary>
            ///     Create an instance of the Operation Queue.
            /// </summary>
            /// 
            /// <param name="displayManager">
            ///     Display manager.
            /// </param>
            function OperationQueue() {
                this.m_bDisposed = null;
                this.m_operationQueue = null;
                this.m_nOperationCount = 0;
                this.m_operationAddedEvent = null;
                this.m_operationQueueSynchronizationObject = null;
                // Create the Operation Queue.
                this.m_operationQueue = new System.Collections.Generic.List();
                // Create the "operation added" manual reset event.
                this.m_operationAddedEvent = new System.Windows.Threading.ManualResetEvent(false);
                // Create the Operation Queue synchronization object.
                /*AIS_TODO this.m_operationQueueSynchronizationObject = new System.Object();*/
            }
            OperationQueue.prototype.Dispose = function () {
                var allParams = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    allParams[_i] = arguments[_i];
                }
                switch (arguments.length) {
                    case 0: return this.Dispose_1();
                    case 1: return this.Dispose_2(arguments[0]);
                    default: throw "Invalid overload call for: Dispose";
                }
            };
            /// <summary>
            ///     Cleanup this object.
            /// </summary>
            OperationQueue.prototype.Dispose_1 = function () {
                // Dispose of this object.
                this.Dispose(true);
                // Take this object off the finalization list.
                /*AIS_TODO GC.SuppressFinalize(this);*/
            };
            /// <summary>
            ///     Clean up.
            /// </summary>
            OperationQueue.prototype.Dispose_2 = function (bDisposing) {
                // If we've already been disposed, simply return.
                if (this.m_bDisposed) {
                    return;
                }
                // Only cleanup/access managed object references if we were called via
                // IDisposable.Dispose( ).
                if (bDisposing) {
                    // Empty the Operation Queue.
                    this.m_operationQueue.Clear();
                    this.m_operationQueue = null;
                    // Explicitly close and release the "operation added" event.
                    this.m_operationAddedEvent.Close();
                    this.m_operationAddedEvent = null;
                }
                // We've been disposed successfully.
                this.m_bDisposed = true;
            };
            //region Public interface.
            /// <summary>
            ///     Add the operation <paramref name="operation"/> to the Operation Queue.
            /// </summary>
            /// 
            /// <param name="operation">
            ///     Operation to add.
            /// </param>
            OperationQueue.prototype.AddOperation = function (operation) {
                // Add the operation to the operation Queue.
                this.m_operationQueue.Add(operation);
                // Increment the operation count.
                this.m_nOperationCount++;
                // Signal the "operation added" event.
                this.m_operationAddedEvent.Set();
            };
            /// <summary>
            ///     Gets the next operation from the queue.
            /// </summary>
            /// 
            /// <returns>
            ///     Operation at the head of the Operation Queue.
            /// </returns>
            OperationQueue.prototype.GetNextOperation = function () {
                // Wait for an operation to be added to the Operation Queue.
                this.WaitForOperation();
                console.assert(this.m_nOperationCount > 0);
                // Decrement the operation count.
                this.m_nOperationCount--;
                var operation = this.m_operationQueue[0];
                this.m_operationQueue.RemoveAt(0);
                // If all the operations have been handled, reset the event.
                if (this.m_nOperationCount == 0) {
                    this.m_operationAddedEvent.Reset();
                }
                return operation;
            };
            /// <summary>
            ///     Add the shutdown operation to the queue.
            /// </summary>
            /// 
            /// <param name="shutdownOperation">
            ///     Shutdown operation.
            /// </param>
            OperationQueue.prototype.AddShutdownOperation = function (shutdownOperation) {
                // Clear the Operation Queue.
                this.m_operationQueue.Clear();
                // Add the shutdown operation to the Operation Queue.
                this.m_nOperationCount = 1;
                this.m_operationQueue.Add(shutdownOperation);
                // Signal the worker thread to shutdown.
                this.m_operationAddedEvent.Set();
            };
            //region Implementation.
            /// <summary>
            ///     Wait for an operation to be added to the Operation Queue.
            /// </summary>
            OperationQueue.prototype.WaitForOperation = function () {
                var operationAddedEvent;
                // Get the "operation added" event.
                operationAddedEvent = this.m_operationAddedEvent;
                console.assert(operationAddedEvent != null);
                // Wait for the "operation added" event to be signalled.
                operationAddedEvent.WaitOne();
            };
            return OperationQueue;
        }());
        Util.OperationQueue = OperationQueue;
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
//------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//     Runtime Version:4.0.30319.296
//
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
//------------------------------------------------------------------------------
// 
// This code was auto-generated by Microsoft.Silverlight.ServiceReference, version 4.0.50826.0
// 
var ViewPoint;
(function (ViewPoint) {
    var Client;
    (function (Client) {
        var Util;
        (function (Util) {
            var HistorianWebService;
            (function (HistorianWebService) {
                var HistorianServiceResultCode;
                (function (HistorianServiceResultCode) {
                    HistorianServiceResultCode[HistorianServiceResultCode["Success"] = 0] = "Success";
                    HistorianServiceResultCode[HistorianServiceResultCode["Faulted"] = 1] = "Faulted";
                    HistorianServiceResultCode[HistorianServiceResultCode["Execption"] = 2] = "Execption";
                    HistorianServiceResultCode[HistorianServiceResultCode["AreaPathNotValid"] = 3] = "AreaPathNotValid";
                    HistorianServiceResultCode[HistorianServiceResultCode["AreaPathDoesNotContainDatalogger"] = 4] = "AreaPathDoesNotContainDatalogger";
                    HistorianServiceResultCode[HistorianServiceResultCode["NoRunningViewApplication"] = 5] = "NoRunningViewApplication";
                    HistorianServiceResultCode[HistorianServiceResultCode["DataRetrievalInternalTimeoutExceeded"] = 6] = "DataRetrievalInternalTimeoutExceeded";
                    HistorianServiceResultCode[HistorianServiceResultCode["StartApplicationTimeoutExceeded"] = 7] = "StartApplicationTimeoutExceeded";
                    HistorianServiceResultCode[HistorianServiceResultCode["WindowsServiceNotAvailable"] = 8] = "WindowsServiceNotAvailable";
                    HistorianServiceResultCode[HistorianServiceResultCode["InvalidParametersList"] = 9] = "InvalidParametersList";
                    HistorianServiceResultCode[HistorianServiceResultCode["RequestedDataNotReady"] = 10] = "RequestedDataNotReady";
                    HistorianServiceResultCode[HistorianServiceResultCode["ConnectionNotExist"] = 11] = "ConnectionNotExist";
                    HistorianServiceResultCode[HistorianServiceResultCode["CreateReaderFailed"] = 12] = "CreateReaderFailed";
                    HistorianServiceResultCode[HistorianServiceResultCode["RetrieveReaderInstanceFailed"] = 13] = "RetrieveReaderInstanceFailed";
                    HistorianServiceResultCode[HistorianServiceResultCode["InvalidDisplayID"] = 14] = "InvalidDisplayID";
                })(HistorianServiceResultCode = HistorianWebService.HistorianServiceResultCode || (HistorianWebService.HistorianServiceResultCode = {}));
                var HistorianWebServiceSoapClient = /** @class */ (function () {
                    function HistorianWebServiceSoapClient(m_host, m_vDir, m_timeout) {
                        this.m_timeout = 10;
                        this.WebProtocol = window.location.protocol;
                        this.GetHistoricalDataCompleted = new Events.Delegate();
                        this.HistoricalRequestCompleted = new Events.Delegate();
                        this.GetRealTimeDataCompleted = new Events.Delegate();
                        this.GetTotalHistoricalTimeSpanCompleted = new Events.Delegate();
                        this.GetRunningViewApplicationCompleted = new Events.Delegate();
                        this.UnInitializeRequestCompleted = new Events.Delegate();
                        this.m_timeout = m_timeout;
                        this.endPointAddress = System.String.Format(CultureInfo.InvariantCulture, "{0}/{1}HistorianServer/Service.asmx", m_host, m_vDir);
                        this.UnInitializeRequestCompleted.subscribe(function () { }, null);
                    }
                    HistorianWebServiceSoapClient.prototype.HistoricalRequestAsync = function (strDisplayName, strDisplayID, strTrendName, strParameters, strClientID, ulStartTime, ulStartTimeMS, ulEndTime, ulEndTimeMS, lResolution, userState) {
                        var data = "<HistoricalRequest xmlns=\"http://rockwell.com/\">";
                        data += "<strDisplayName>" + strDisplayName + "</strDisplayName >";
                        data += "<strDisplayID>" + strDisplayID + "</strDisplayID >";
                        data += "<strTrendName>" + strTrendName + "</strTrendName >";
                        data += "<strParameters>" + strParameters + "</strParameters >";
                        data += "<strClientID>" + strClientID + "</strClientID >";
                        data += "<ulStartTime>" + ulStartTime + "</ulStartTime >";
                        data += "<ulStartTimeMS>" + ulStartTimeMS + "</ulStartTimeMS >";
                        data += "<ulEndTime>" + ulEndTime + "</ulEndTime >";
                        data += "<ulEndTimeMS>" + ulEndTimeMS + "</ulEndTimeMS >";
                        data += "<lResolution>" + lResolution + "</lResolution>";
                        data += "</HistoricalRequest></soap:Body></soap:Envelope> ";
                        WebServiceUtils.CallWebService(data, this.endPointAddress, "http://rockwell.com/HistoricalRequest", "HistoricalRequestResponse", this.HistoricalRequestCompleted, "HistoricalRequest", userState);
                    };
                    HistorianWebServiceSoapClient.prototype.GetRealTimeDataAsync = function (strDisName, strDispID, strPara, userState) {
                        var data = "<GetRealTimeData xmlns=\"http://rockwell.com/\">";
                        data += "<strDisName>" + strDisName + "</strDisName >";
                        data += "<strDispID>" + strDispID + "</strDispID >";
                        data += "<strPara>" + strPara + "</strPara >";
                        data += "</GetRealTimeData></soap:Body></soap:Envelope> ";
                        WebServiceUtils.CallWebService(data, this.endPointAddress, "http://rockwell.com/GetRealTimeData", "GetRealTimeDataResponse", this.GetRealTimeDataCompleted, "GetRealTimeData", userState);
                    };
                    HistorianWebServiceSoapClient.prototype.GetHistoricalDataAsync = function (strClientID, bMultiPens, userState) {
                        var data = "<GetHistoricalData xmlns=\"http://rockwell.com/\">";
                        data += "<strClientID>" + strClientID + "</strClientID >";
                        data += "<bMultiPens>" + bMultiPens + "</bMultiPens >";
                        data += "</GetHistoricalData></soap:Body></soap:Envelope> ";
                        WebServiceUtils.CallWebService(data, this.endPointAddress, "http://rockwell.com/GetHistoricalData", "GetHistoricalDataResponse", this.GetHistoricalDataCompleted, "GetHistoricalData", userState);
                    };
                    HistorianWebServiceSoapClient.prototype.GetTotalHistoricalTimeSpanAsync = function (strClientID, userState) {
                        var data = "<GetTotalHistoricalTimeSpan xmlns=\"http://rockwell.com/\">";
                        data += "<strClientID>" + strClientID + "</strClientID >";
                        data += "</GetTotalHistoricalTimeSpan></soap:Body></soap:Envelope> ";
                        WebServiceUtils.CallWebService(data, this.endPointAddress, "http://rockwell.com/GetTotalHistoricalTimeSpan", "GetTotalHistoricalTimeSpanResponse", this.GetTotalHistoricalTimeSpanCompleted, "GetTotalHistoricalTimeSpan", userState);
                    };
                    HistorianWebServiceSoapClient.prototype.UnInitializeRequestAsync = function (strClientID) {
                        var data = "<UnInitializeRequest xmlns=\"http://rockwell.com/\">";
                        data += "<strClientID>" + strClientID + "</strClientID >";
                        data += "</UnInitializeRequest></soap:Body></soap:Envelope> ";
                        WebServiceUtils.CallWebService(data, this.endPointAddress, "http://rockwell.com/UnInitializeRequest", "UnInitializeRequestResponse", this.UnInitializeRequestCompleted, "UnInitializeRequestResponse", null);
                    };
                    return HistorianWebServiceSoapClient;
                }());
                HistorianWebService.HistorianWebServiceSoapClient = HistorianWebServiceSoapClient;
            })(HistorianWebService = Util.HistorianWebService || (Util.HistorianWebService = {}));
        })(Util = Client.Util || (Client.Util = {}));
    })(Client = ViewPoint.Client || (ViewPoint.Client = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        var CommonUtilities = /** @class */ (function () {
            function CommonUtilities() {
            }
            /// <summary>
            ///     Checks to see if cookies are enabled in this browser.
            /// </summary>
            /// 
            /// <returns>
            ///     Returns "true" if cookies are eanbled, "false" if not.
            /// </returns>
            CommonUtilities.CookiesEnabled = function () {
                try {
                    var cookieEnabled;
                    if (CurrentBrowser.browserName() == 'Chrome') {
                        cookieEnabled = (navigator.cookieEnabled) ? true : false;
                    }
                    else {
                        document.cookie = "ftvptestcookie=yes";
                        cookieEnabled = (document.cookie.indexOf("ftvptestcookie") != -1) ? true : false;
                    }
                    return (cookieEnabled);
                }
                catch (_) {
                    return false;
                }
            };
            CommonUtilities.IsNotSupportCookieEnabledProperty = function () {
                return typeof navigator.cookieEnabled == "undefined";
            };
            /* AIS_NOTE: Unused function
            /// <summary>
            ///     Searches the visual tree for a FrameworkElement with a given name.
            /// </summary>
            ///
            /// <returns>
            ///     Returns the found 'FrameworkElement' or null
            /// </returns>
            ///
            /// <remarks>
            ///     This method will take some time given a parent element with many nested children
            /// </remarks>
            static GetTemplateChildByName(parent,name) {
                var childnum = VisualTreeHelper.GetChildrenCount(parent);
                for(var i = 0;i < childnum;i++) {
                    var child = VisualTreeHelper.GetChild(parent,i);
                    var feChild = helpers.asType(child,System.Windows.FrameworkElement);
                    if ((null != feChild) && (feChild.getName() == name)) {
                        return feChild;
                    } else {
                        var s = ViewPoint.Util.CommonUtilities.GetTemplateChildByName(child,name);
                        if (s != null)return s;
                    }
                }
                return null;
            }*/
            /// <summary>
            ///     For trend use,covert the datetime to datalogtime
            /// </summary>
            /// 
            /// <returns>
            ///     Returns datalogtime
            /// </returns>
            CommonUtilities.DateTimeToDatalogTime = function (theDateTime) {
                var tm1970 = System.DateTime.GetTm1970(theDateTime);
                var subtract = theDateTime.Date.valueOf() - tm1970;
                return Math.round(subtract / 1000);
            };
            /// <summary>
            ///     For trend use,covert the datalogtime to datetime
            /// </summary>
            /// 
            /// <returns>
            ///     Returns datetime
            /// </returns>
            CommonUtilities.DatalogTimeToDateTime = function (dlgtimeHi, dlgtimeLo) {
                var bigTicks = dlgtimeHi * 1000;
                var theDateTime = new System.DateTime(bigTicks).ToUniversalTime();
                var tm1970 = System.DateTime.GetTm1970(theDateTime);
                var adjTime = new System.DateTime(tm1970 + bigTicks);
                return adjTime;
            };
            CommonUtilities.ParseTime = function (timeticks) {
                // the number of .net ticks at the unix epoch
                var epochTicks = 621355968000000000;
                // there are 10000 .net ticks per millisecond
                var ticksPerMillisecond = 10000;
                // calculate the total number of .net ticks for your date
                var ticks = (timeticks - epochTicks) / ticksPerMillisecond;
                var datum = new Date(ticks);
                return new System.DateTime(datum);
            };
            CommonUtilities.CalculateDraggableArea = function (dlg) {
                try {
                    var x1, x2, y1, y2;
                    var VpBanner = $("#_vpBanner");
                    var VpBannerHeight = VpBanner.height();
                    var StatusBar = $("#StatusBar");
                    var StatusBarHeight = StatusBar.height();
                    var Splitter = $("#GridSplitter .hsplitter");
                    var SplitterHeight = Splitter.height();
                    x1 = 0;
                    y1 = VpBannerHeight;
                    x2 = $(window).width() - dlg.outerWidth();
                    var DisplayAreaHeight = $(window).height() - StatusBarHeight - SplitterHeight;
                    if (DisplayAreaHeight < dlg.outerHeight()) {
                        y2 = VpBannerHeight + dlg.outerHeight();
                    }
                    else {
                        y2 = $(window).height() - StatusBarHeight - SplitterHeight - dlg.outerHeight();
                    }
                    return [x1, y1, x2, y2];
                }
                catch (e) {
                    return $("#DisplayArea");
                }
            };
            /// <summary>
            ///     Replace decimal separator to current culture format.
            /// </summary>
            /// 
            /// <returns>
            ///     Returns converted string value in current culture.
            /// </returns>
            CommonUtilities.ReplaceSeparator = function (convertValue) {
                var separator = System.Globalization.CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator;
                if (separator != ".") {
                    convertValue = convertValue.replace(".", separator);
                }
                return convertValue;
            };
            return CommonUtilities;
        }());
        Util.CommonUtilities = CommonUtilities;
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        var SerializerType;
        (function (SerializerType) {
            SerializerType[SerializerType["JSON"] = 0] = "JSON";
            SerializerType[SerializerType["XML"] = 1] = "XML";
        })(SerializerType = Util.SerializerType || (Util.SerializerType = {}));
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        var StreamEncodingFormat;
        (function (StreamEncodingFormat) {
            StreamEncodingFormat[StreamEncodingFormat["Text"] = 0] = "Text";
            StreamEncodingFormat[StreamEncodingFormat["Binary"] = 1] = "Binary";
        })(StreamEncodingFormat = Util.StreamEncodingFormat || (Util.StreamEncodingFormat = {}));
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
/// <reference path="./SerializerType.ts" />
/// <reference path="./StreamEncodingFormat.ts" />
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        var Serializer = /** @class */ (function () {
            function Serializer() {
            }
            Serializer.Serialize = function (obj) {
                /*AIS-TODO if (Serializer.FormatType == SerializerType.JSON) return JSON.Serialize(obj);
                var ms : MemoryStream = new System.IO.MemoryStream();
                return Encoding.UTF8.GetString(ms.ToArray(), 0, ms.ToArray().length); */
                return "";
            };
            Serializer.Deserialize = function (text) {
                if (Serializer.FormatType == Util.SerializerType.JSON)
                    return jQuery.parseJSON(text);
            };
            Serializer.FormatType = Util.SerializerType.JSON;
            Serializer.FormatEncoding = Util.StreamEncodingFormat.Text;
            Serializer.staticConstructor = (function () {
                Serializer.FormatType = Util.SerializerType.JSON;
                Serializer.FormatEncoding = Util.StreamEncodingFormat.Text;
            })();
            return Serializer;
        }());
        Util.Serializer = Serializer;
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        var List = System.Collections.Generic.List;
        var ReadDataResponse = /** @class */ (function () {
            function ReadDataResponse() {
                this.Pens = new List();
                this.RequestID = null;
                this.MoreData = null;
                this.Pens = new System.Collections.Generic.List();
            }
            return ReadDataResponse;
        }());
        Util.ReadDataResponse = ReadDataResponse;
        var PenDataList = /** @class */ (function () {
            function PenDataList() {
                this.Items = new List();
                this.ServerName = null;
                this.TagGroup = null;
                this.PenID = null;
                this.Items = new System.Collections.Generic.List();
            }
            return PenDataList;
        }());
        Util.PenDataList = PenDataList;
        var PenDataItem = /** @class */ (function () {
            function PenDataItem() {
                this.Time = null;
                this.Value = null;
                this.Properties = null;
            }
            Object.defineProperty(PenDataItem.prototype, "TimeAsDateTime", {
                get: function () {
                    return PenDataItem.DataLogTimeToDateTime(this.Time);
                },
                enumerable: true,
                configurable: true
            });
            PenDataItem.DataLogTimeToDateTime = function (dataLogTime) {
                var theDateTime = new System.DateTime(dataLogTime).ToUniversalTime();
                var tm1970 = System.DateTime.GetTm1970(theDateTime);
                var adjTime = new System.DateTime(tm1970 + dataLogTime);
                return adjTime;
            };
            return PenDataItem;
        }());
        Util.PenDataItem = PenDataItem;
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        var Trace = /** @class */ (function () {
            function Trace() {
            }
            Object.defineProperty(Trace, "IndentLevel", {
                /// <summary>
                /// Gets or sets the indent level.
                /// </summary>
                get: function () {
                    return Trace.indentLevel;
                },
                /// <summary>
                /// Gets or sets the indent level.
                /// </summary>
                set: function (value) {
                    Trace.indentLevel = value;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Trace, "Destination", {
                get: function () {
                    Trace.destination = HtmlPage.Document.GetElementById(Trace.DestinationId);
                    if (Trace.destination == null) {
                        Trace.destination = HtmlPage.Document.CreateElement("div");
                        /*AIS_TODO Trace.destination.SetProperty("id", Trace.DestinationId)*/ ;
                        //make it invisible. Not being used currently.
                        //destination.SetStyleAttribute("visibility", "hidden");
                        HtmlPage.Document.Body.AppendChild(Trace.destination);
                    }
                    return Trace.destination;
                },
                enumerable: true,
                configurable: true
            });
            ///// <summary>
            ///// constructor
            ///// </summary>
            ////Removed to get rid of CA1810 Warning
            //static Trace()
            //{
            //    if (Application.Current is ITracing)
            //    {
            //        isTracingOn = ((ITracing)Application.Current).IsTracingOn;
            //        isWrittenToFile = ((ITracing)Application.Current).IsTracingWrittenToFile;
            //    }
            //}
            Trace.InitializeTracingFlag = function () {
                try {
                    if (lang.isType(Application.Current)) {
                        return Application.Current.IsTracingOn;
                    }
                    else {
                        return false;
                    }
                }
                catch (e) {
                    console.log("Trace InitializeTracingFlag has thrown an exception.");
                }
            };
            Trace.InitializeWriteToFileFlag = function () {
                try {
                    if (lang.isType(Application.Current)) {
                        return Application.Current.IsTracingWrittenToFile;
                    }
                    else {
                        return false;
                    }
                }
                catch (e) {
                    console.log("Trace InitializeTracingFlag has thrown an exception.");
                }
            };
            /// <summary>
            /// Flush the buffer data to destination
            /// </summary>
            Trace.Flush = function () {
                if (Trace.isTracingOn) {
                    if (Trace.callsSinceLastFlush == 0)
                        return;
                    if (Trace.isWrittenToFile) {
                        Trace.WriteBufferDataToFile();
                    }
                    else {
                        Trace.WriteBufferDataToPage();
                    }
                }
            };
            /// <summary>
            /// Write the message to destination if the condition is true
            /// </summary>
            /// <param name="condition"></param>
            /// <param name="message"></param>
            Trace.WriteIf = function (condition, message) {
                if (condition) {
                    Trace.Write(message);
                }
            };
            /// <summary>
            /// Writes a message to the trace destination
            /// </summary>
            /// <param name="message"></param>
            Trace.Write = function (message) {
                if (Trace.isTracingOn) {
                    //Write to both if the flag is on
                    if (Trace.isWrittenToFile) {
                        Trace.pageBuffer.Append(Trace.GetSeparator(false) + message);
                        Trace.fileBuffer.Append(Trace.GetSeparator(true) + message);
                        Trace.callsSinceLastFlush++;
                        if (Trace.callsSinceLastFlush < Trace.CallsBetweenFlush)
                            return;
                        Trace.WriteBufferDataToPage();
                        Trace.WriteBufferDataToFile();
                    }
                    else {
                        Trace.pageBuffer.Append(Trace.GetSeparator(false) + message);
                        Trace.callsSinceLastFlush++;
                        if (Trace.callsSinceLastFlush < Trace.CallsBetweenFlush)
                            return;
                        Trace.WriteBufferDataToPage();
                    }
                }
            };
            /// <summary>
            ///  Write the message to destination if the condition is true
            /// </summary>
            /// <param name="condition"></param>
            /// <param name="message"></param>
            Trace.WriteLineIf = function (condition, message) {
                if (condition) {
                    Trace.WriteLine(message);
                }
            };
            /// <summary>
            /// Writes a message to the trace destination
            /// </summary>
            /// <param name="message"></param>
            Trace.WriteLine = function (message) {
                if (Trace.isTracingOn) {
                    if (Trace.isWrittenToFile) {
                        Trace.fileBuffer.Append(Trace.GetSeparator(true));
                        Trace.fileBuffer.Append(System.DateTime.Now.ToString(Trace.DateTimeFormat, Trace.formatProvider));
                        Trace.fileBuffer.Append(Trace.GetSeparator(true, 2));
                        Trace.fileBuffer.Append(message + "\r\n");
                        Trace.pageBuffer.Append(Trace.GetSeparator(false));
                        Trace.pageBuffer.Append(System.DateTime.Now.ToString(Trace.DateTimeFormat, Trace.formatProvider));
                        Trace.pageBuffer.Append(Trace.GetSeparator(false, 2));
                        Trace.pageBuffer.Append(message);
                        Trace.pageBuffer.Append("<br>");
                        Trace.callsSinceLastFlush++;
                        if (Trace.callsSinceLastFlush < Trace.CallsBetweenFlush)
                            return;
                        Trace.WriteBufferDataToPage();
                        Trace.WriteBufferDataToFile();
                    }
                    else {
                        Trace.pageBuffer.Append(Trace.GetSeparator(false));
                        Trace.pageBuffer.Append(System.DateTime.Now.ToString(Trace.DateTimeFormat, Trace.formatProvider));
                        Trace.pageBuffer.Append(Trace.GetSeparator(false, 2));
                        Trace.pageBuffer.Append(message);
                        Trace.pageBuffer.Append("<br>");
                        Trace.callsSinceLastFlush++;
                        if (Trace.callsSinceLastFlush < Trace.CallsBetweenFlush)
                            return;
                        Trace.WriteBufferDataToPage();
                    }
                }
            };
            //region private static helpers
            /// <summary>
            /// Write the buffer data to a html page
            /// </summary>
            Trace.WriteBufferDataToPage = function () {
                var innerHtml /*AIS_TODO = Trace.Destination.GetProperty("innerHTML").toString()*/;
                /*AIS_TODO Trace.Destination.SetProperty("innerHTML", innerHtml + Trace.pageBuffer.ToString());*/
                Trace.callsSinceLastFlush = 0;
                Trace.pageBuffer.Remove(0, Trace.pageBuffer.Length);
            };
            /// <summary>
            /// Write the data to a file in Isolated Storage
            /// </summary>
            Trace.WriteBufferDataToFile = function () {
                var isf = IsolatedStorageFile.GetUserStoreForApplication();
                try {
                    var isfs = null;
                    if (isf.FileExists(Trace.TraceLogFile)) {
                        if (isf.AvailableFreeSpace < Trace.fileBuffer.Length) {
                            //Truncate the file and start it over. 
                            isfs = new IsolatedStorageFileStream(Trace.TraceLogFile, FileMode.Truncate, FileAccess.Write, isf);
                        }
                        else {
                            isfs = new IsolatedStorageFileStream(Trace.TraceLogFile, FileMode.Append, FileAccess.Write, isf);
                        }
                    }
                    else {
                        if (isf.AvailableFreeSpace < Trace.fileBuffer.Length) {
                            //send the message to the browser. Can't call IncreaseQuotaTo here. It could be called 
                            //in the user events (such as button click) only.
                            Trace.isWrittenToFile = false;
                            return;
                        }
                        else {
                            isfs = new IsolatedStorageFileStream(Trace.TraceLogFile, FileMode.Create, isf);
                        }
                    }
                    var sw = new StreamWriter(isfs);
                    try {
                        sw.Write(Trace.fileBuffer.ToString());
                        sw.Close();
                    }
                    finally {
                        /*AIS_TODO sw.Dispose();*/
                    }
                    isfs.Dispose();
                    Trace.callsSinceLastFlush = 0;
                    Trace.fileBuffer.Remove(0, Trace.fileBuffer.Length);
                }
                finally {
                    isf.Dispose();
                }
            };
            Trace.GetSeparator = function () {
                var allParams = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    allParams[_i] = arguments[_i];
                }
                switch (arguments.length) {
                    case 1: return ViewPoint.Util.Trace.GetSeparator_1(arguments[0]);
                    case 2: return ViewPoint.Util.Trace.GetSeparator_2(arguments[0], arguments[1]);
                    default: throw "Invalid overload call for: GetSeparator";
                }
            };
            Trace.GetSeparator_1 = function (isWrittenToFile) {
                var separator;
                if (Trace.isWrittenToFile) {
                    separator = " ";
                }
                else {
                    separator = "&nbsp;";
                }
                var value = "";
                for (var i = 0; i < Trace.IndentLevel; i++)
                    value += separator;
                return value;
            };
            Trace.GetSeparator_2 = function (isWrittenToFile, count) {
                var separator;
                if (Trace.isWrittenToFile) {
                    separator = " ";
                }
                else {
                    separator = "&nbsp;";
                }
                var value = "";
                for (var i = 0; i < count; i++)
                    value += separator;
                return value;
            };
            Trace.TraceLogFile = "ViewPointTracing.log";
            Trace.CallsBetweenFlush = 16;
            Trace.DestinationId = "ViewPointTraceOutput";
            Trace.DateTimeFormat = "MM/dd/yyyy HH:mm:ss:fff";
            Trace.isTracingOn = Trace.InitializeTracingFlag();
            Trace.isWrittenToFile = Trace.InitializeWriteToFileFlag();
            Trace.pageBuffer = new System.Text.StringBuilder(1024);
            Trace.fileBuffer = new System.Text.StringBuilder(1024);
            Trace.formatProvider = new CultureInfo("en-US");
            Trace.staticConstructor = (function () {
                Trace.isTracingOn = Trace.InitializeTracingFlag();
                Trace.isWrittenToFile = Trace.InitializeWriteToFileFlag();
                Trace.callsSinceLastFlush = 0;
                Trace.pageBuffer = new System.Text.StringBuilder(1024);
                Trace.fileBuffer = new System.Text.StringBuilder(1024);
                Trace.formatProvider = new System.Globalization.CultureInfo("en-US");
                Trace.syncRoot /*AIS_TODO = new System.Object()*/;
                Trace.indentLevel = 0;
                Trace.destination = null;
            })();
            return Trace;
        }());
        Util.Trace = Trace;
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
var ViewPoint;
(function (ViewPoint) {
    var Util;
    (function (Util) {
        var ViewPointHelperClient = /** @class */ (function () {
            function ViewPointHelperClient() {
            }
            /// <summary>
            /// Returns the HttpBinding on different scheme.
            /// </summary>
            /// <returns>An instance of httpBinding</returns>
            ViewPointHelperClient.GetBasicHttpBinding = function (uriScheme) {
                var httpBinding = null;
                if (System.String.Compare(uriScheme, /*AIS_TODO BasicHttpSecurityMode*/ "HTTPS", System.StringComparison.OrdinalIgnoreCase) == 0) {
                    httpBinding = new BasicHttpBinding(BasicHttpSecurityMode.Transport);
                }
                else {
                    httpBinding = new BasicHttpBinding();
                }
                return httpBinding;
            };
            return ViewPointHelperClient;
        }());
        Util.ViewPointHelperClient = ViewPointHelperClient;
    })(Util = ViewPoint.Util || (ViewPoint.Util = {}));
})(ViewPoint || (ViewPoint = {}));
//# sourceMappingURL=ViewPoint.Client.Util.js.map